@@ -27,28 +27,36 @@
 // TODO: Take care of Libraries(gnustep-gui, gnustep-base) 
 //       and Non Project Files
 
-#include <ProjectCenter/PCFileManager.h>
-#include <ProjectCenter/PCProjectManager.h>
-#include <ProjectCenter/PCProject.h>
-#include <ProjectCenter/PCDefines.h>
+#import <ProjectCenter/PCFileManager.h>
+#import <ProjectCenter/PCProjectManager.h>
+#import <ProjectCenter/PCProject.h>
+#import <ProjectCenter/PCDefines.h>
 
-#include <ProjectCenter/PCProjectWindow.h>
-#include <ProjectCenter/PCProjectBrowser.h>
-#include <ProjectCenter/PCProjectLoadedFiles.h>
+#import <ProjectCenter/PCProjectWindow.h>
+#import <ProjectCenter/PCProjectBrowser.h>
+#import <ProjectCenter/PCProjectLoadedFiles.h>
 
-#include <ProjectCenter/PCProjectInspector.h>
-#include <ProjectCenter/PCProjectBuilder.h>
-#include <ProjectCenter/PCProjectEditor.h>
-#include <ProjectCenter/PCProjectLauncher.h>
+#import <ProjectCenter/PCProjectInspector.h>
+#import <ProjectCenter/PCProjectBuilder.h>
+#import <ProjectCenter/PCProjectEditor.h>
+#import <ProjectCenter/PCProjectLauncher.h>
 
-#include <ProjectCenter/PCLogController.h>
+#import <ProjectCenter/PCLogController.h>
 
-#include <Protocols/CodeEditor.h>
+#import <Protocols/CodeEditor.h>
+
+#import "Modules/Preferences/Saving/PCSavingPrefs.h"
+#import "Modules/Preferences/Misc/PCMiscPrefs.h"
+
+// #import <AppKit/NSFileWrapper.h>
+// #import <Foundation/NSData.h>
 
 NSString 
 *PCProjectDictDidChangeNotification = @"PCProjectDictDidChangeNotification";
 NSString 
 *PCProjectDictDidSaveNotification = @"PCProjectDictDidSaveNotification";
+NSString 
+*PCProjectBreakpointNotification = @"PCProjectBreakpointNotification";
 
 @implementation PCProject
 
@@ -76,18 +84,52 @@ NSString
   return self;
 }
 
-- (PCProject *)openWithDictionaryAt:(NSString *)path
+- (PCProject *)openWithWrapperAt:(NSString *)aPath
 {
-  NSDictionary *dict = [NSDictionary dictionaryWithContentsOfFile:path];
-
-  [self assignProjectDict:dict atPath:path];
-
+  BOOL isDir = NO;
+  BOOL exists = [[NSFileManager defaultManager] fileExistsAtPath: aPath 
+						isDirectory: &isDir];
+  if(isDir && exists)
+    {
+      projectFileWrapper = [[NSFileWrapper alloc] initWithPath: aPath];
+      if(projectFileWrapper != nil)
+	{
+	  NSDictionary *wrappers = [projectFileWrapper fileWrappers];
+	  NSData *data = [[wrappers objectForKey: @"PC.project"] regularFileContents];
+	  NSData *userData = [[wrappers objectForKey: [NSUserName() stringByAppendingPathExtension: @"project"]]
+			       regularFileContents];
+	  NSMutableDictionary *dict = [[[[NSString alloc] initWithData: data
+							  encoding: NSASCIIStringEncoding] 
+					 propertyList] mutableCopy];
+	  NSDictionary *udict = [[[NSString alloc] initWithData: userData
+						   encoding: NSASCIIStringEncoding] 
+				  propertyList];
+	  
+	  [dict addEntriesFromDictionary: udict]; 
+	  [self assignProjectDict:dict atPath: aPath];
+	}
+    }
+  else
+    {
+      NSMutableDictionary *dict = [NSMutableDictionary dictionaryWithContentsOfFile: aPath];
+      
+      projectFileWrapper = [[NSFileWrapper alloc] 
+			     initDirectoryWithFileWrappers: 
+			       [NSMutableDictionary dictionaryWithCapacity: 3]];
+      [projectFileWrapper addRegularFileWithContents:
+			    [NSData dataWithBytes: [[dict description] cString]
+				    length: [[dict description] length]]
+			  preferredFilename: @"PC.project"];
+      [self assignProjectDict: dict
+	    atPath: aPath];
+    }
+  
   return self;
 }
 
 - (void)dealloc
 {
-#ifdef DEVELOPMENT
+#ifdef DEBUG
   NSLog (@"PCProject %@: dealloc", projectName);
 #endif
   
@@ -105,8 +147,8 @@ NSString
   RELEASE(projectBrowser);
   RELEASE(projectLoadedFiles);
   RELEASE(projectEditor);
-  if (projectBuilder) RELEASE(projectBuilder);
-  if (projectLauncher) RELEASE(projectLauncher);
+  TEST_RELEASE(projectBuilder);
+  TEST_RELEASE(projectLauncher);
 
   if (isSubproject == YES)
     {
@@ -117,6 +159,14 @@ NSString
   [super dealloc];
 }
 
+- (void)loadPreferences:(NSNotification *)aNotification
+{
+  id <PCPreferences> prefs = [projectManager prefController];
+
+  rememberWindows = [prefs boolForKey:RememberWindows];
+  keepBackup = [prefs boolForKey:KeepBackup];
+}
+
 // ============================================================================
 // ==== Project handling
 // ============================================================================
@@ -135,7 +185,8 @@ NSString
   projectDict = [[NSMutableDictionary alloc] initWithDictionary:pDict];
 
   // Project path
-  if ([[pPath lastPathComponent] isEqualToString:@"PC.project"])
+  if ([[pPath lastPathComponent] isEqualToString:@"PC.project"] ||
+      [[[pPath lastPathComponent] pathExtension] isEqualToString:@"pcproj"])
     {
       [self setProjectPath:[pPath stringByDeletingLastPathComponent]];
     }
@@ -186,8 +237,10 @@ NSString
     {
       [self updateProjectDict];
 
-      NSRunAlertPanel(@"Project updated!", 
-		      @"The project file was converted from previous version!\nPlease make sure that every project attribute contain valid values!", 
+      NSRunAlertPanel(@"Open Project!", 
+		      @"The project file was converted from previous version!\n"
+		      @"Please make sure that every project attribute contain"
+		      @" valid values!", 
 		      @"OK",nil,nil);
     }
 }
@@ -315,14 +368,12 @@ NSString
 
 - (BOOL)saveProjectWindowsAndPanels
 {
-  NSUserDefaults      *ud = [NSUserDefaults standardUserDefaults];
   NSMutableDictionary *windows = [NSMutableDictionary dictionary];
   NSString            *projectFile = nil;
   NSMutableDictionary *projectFileDict = nil;
 
-  projectFile = [projectPath stringByAppendingPathComponent:@"PC.project"];
-  projectFileDict = [NSMutableDictionary 
-    dictionaryWithContentsOfFile:projectFile];
+  projectFile = [NSUserName() stringByAppendingPathExtension:@"project"];
+  projectFileDict = [[NSMutableDictionary alloc] initWithCapacity:4];
 
   // Project Window
   [windows setObject:[projectWindow stringWithSavedFrame]
@@ -342,15 +393,14 @@ NSString
   [windows setObject:NSStringFromRect([[projectBrowser view] frame])
               forKey:@"ProjectBrowser"];
 
-  // Write to file and exit if prefernces wasn't set to save panels
-  if (![[ud objectForKey:RememberWindows] isEqualToString:@"YES"])
+  // Write to file and exit if preferences wasn't set to save panels
+  if (!rememberWindows)
     {
-      [projectFileDict setObject:windows forKey:@"PC_WINDOWS"];
+      [projectFileDict setObject:windows forKey:PCWindows];
       [projectFileDict writeToFile:projectFile atomically:YES];
       return YES;
     }
 
-
   // Project Build
   if (projectBuilder && [[projectManager buildPanel] isVisible])
     {
@@ -373,17 +423,6 @@ NSString
       [windows removeObjectForKey:@"ProjectLaunch"];
     }
 
-  // Project Inspector
-/*  if ([[projectManager inspectorPanel] isVisible])
-    {
-      [windows setObject:[[projectManager inspectorPanel] stringWithSavedFrame]
-                  forKey:@"ProjectInspector"];
-    }
-  else
-    {
-      [windows removeObjectForKey:@"ProjectInspector"];
-    }*/
-
   // Loaded Files
   if (projectLoadedFiles && [[projectManager loadedFilesPanel] isVisible])
     {
@@ -398,28 +437,50 @@ NSString
 
   // Set to project dict for case if project changed
   // Don't notify about projectDict changes
-  [projectDict setObject:windows forKey:@"PC_WINDOWS"];
+  [projectDict setObject:windows forKey:PCWindows];
 
-  // Now save it directly to PC.project file
-  [projectFileDict setObject:windows forKey:@"PC_WINDOWS"];
-  [projectFileDict writeToFile:projectFile atomically:YES];
+  // Now save it directly to username.project file
+  [projectFileDict setObject:windows forKey:PCWindows];
+
+  [projectFileDict setObject: [[NSCalendarDate date] description]
+	       forKey: PCLastEditing];
+
+  // add the file and write the wrapper.
+  [projectFileWrapper addRegularFileWithContents:
+			[NSData dataWithBytes: [[projectFileDict description] cString]
+				length: [[projectFileDict description] length]]
+		      preferredFilename: projectFile];
+  [projectFileWrapper writeToFile: wrapperPath 
+		      atomically: YES 
+		      updateFilenames: YES];
   
-//  PCLogInfo(self, @"Windows and geometries saved");
+  // release
+  [projectFileDict release];
 
   return YES;
 }
 
 - (BOOL)save
 {
-  NSString *file = [projectPath stringByAppendingPathComponent:@"PC.project"];
-  NSString       *backup = [file stringByAppendingPathExtension:@"backup"];
   NSFileManager  *fm = [NSFileManager defaultManager];
-  NSUserDefaults *defs = [NSUserDefaults standardUserDefaults];
-  NSString       *keepBackup = [defs objectForKey:KeepBackup];
-  BOOL           shouldKeep = [keepBackup isEqualToString:@"YES"];
   int            spCount = [loadedSubprojects count];
   int            i;
+  NSString *wrapperFile = [projectName stringByAppendingPathExtension: @"pcproj"];
+  NSString *file = @"PC.project";
+  NSString *backup = [wrapperPath stringByAppendingPathExtension:@"backup"];
+  NSMutableDictionary *dict = [projectDict mutableCopy];
+  NSData *dictData = nil;
 
+  // remove key..
+  [dict removeObjectForKey: PCWindows];
+  [dict removeObjectForKey: PCLastEditing];
+
+  // initialize the wrapper...
+  projectFileWrapper = [[NSFileWrapper alloc] initDirectoryWithFileWrappers: 
+						[NSMutableDictionary dictionaryWithCapacity: 3]];  
+  wrapperPath = [projectPath stringByAppendingPathComponent: wrapperFile];
+
+  // load subprojects...
   for (i = 0; i < spCount; i++)
     {
       [[loadedSubprojects objectAtIndex:i] save];
@@ -428,29 +489,40 @@ NSString
   // Remove backup file if exists
   if ([fm fileExistsAtPath:backup] && ![fm removeFileAtPath:backup handler:nil])
     {
-      NSRunAlertPanel(@"Save project",
-		      @"Error removing the old project backup!",
+      NSRunAlertPanel(@"Save Project",
+		      @"Couldn't remove the old project backup file",
 		      @"OK",nil,nil);
       return NO;
     }
 
   // Save backup
-  if (shouldKeep == YES && [fm isReadableFileAtPath:file]) 
+  if ((keepBackup == YES) && [fm isReadableFileAtPath: wrapperPath]) 
     {
-      if ([fm copyPath:file toPath:backup handler:nil] == NO)
+      if ([fm copyPath: wrapperPath 
+	      toPath: backup 
+	      handler:nil] == NO)
 	{
-	  NSRunAlertPanel(@"Save project",
-			  @"Error when saving project backup file!",
+	  NSRunAlertPanel(@"Save Project",
+			  @"Couldn't save project backup file",
 			  @"OK",nil,nil);
 	  return NO;
 	}
     }
 
   // Save project file
-  [projectDict setObject:[[NSCalendarDate date] description]
-                  forKey:PCLastEditing];
-  if ([projectDict writeToFile:file atomically:YES] == NO)
+  dictData = [NSPropertyListSerialization dataFromPropertyList: dict
+					  format: NSPropertyListOpenStepFormat
+					  errorDescription: NULL];
+  [projectFileWrapper addRegularFileWithContents: dictData
+		      preferredFilename: file];
+  if ([projectFileWrapper
+	writeToFile:wrapperPath
+	atomically:YES 
+	updateFilenames: YES] == NO)
     {
+      NSRunAlertPanel(@"Save Project",
+		      @"Couldn't save project file",
+		      @"OK",nil,nil,projectName);
       return NO;
     }
 
@@ -461,9 +533,9 @@ NSString
   // Save GNUmakefile
   if ([self writeMakefile] == NO)
     {
-      NSRunAlertPanel(@"Save project",
-		      @"Error when writing makefile for project %@",
-		      @"OK",nil,nil,projectName);
+      NSRunAlertPanel(@"Save Project",
+		      @"Couldn't write GNUmakefile",
+		      @"OK",nil,nil);
       return NO;
     }
 
@@ -487,9 +559,9 @@ NSString
     {
       int ret;
 
-      ret = NSRunAlertPanel(@"Alert",
+      ret = NSRunAlertPanel(@"Close Project",
 			    @"Project or subprojects are modified",
-			    @"Save and Close",@"Don't save",@"Cancel");
+			    @"Save and Close",@"Don't save",@"Stop");
       switch (ret)
 	{
 	case NSAlertDefaultReturn:
@@ -529,12 +601,6 @@ NSString
       return NO;
     }
 
-  // Project Builder
-/*  if (projectBuilder)
-    {
-      [projectBuilder release];
-    }*/
-  
   // Project window
   if (sender != projectWindow)
     {
@@ -586,6 +652,13 @@ NSString
     {
       projectWindow = [[PCProjectWindow alloc] initWithProject:self];
     }
+
+  [[NSNotificationCenter defaultCenter]
+    addObserver:self
+       selector:@selector(loadPreferences:)
+	   name:PCPreferencesDidChangeNotification
+	 object:nil];
+  [self loadPreferences:nil];
 }
 
 - (PCProjectWindow *)projectWindow
@@ -680,14 +753,21 @@ NSString
   return NO;
 }
 
-- (NSString *)execToolName
-{
-  return [[self projectBundleInfoTable] objectForKey:@"ExecToolName"];
-}
 
 - (NSArray *)buildTargets
 {
-  return [[self projectBundleInfoTable] objectForKey:@"BuildTargets"];
+  NSArray *buildTargets = [projectDict objectForKey:PCBuilderTargets];
+
+  if (!buildTargets)
+    {
+      buildTargets = 
+	[[self projectBundleInfoTable] objectForKey:@"BuildTargets"];
+      [self setProjectDictObject:buildTargets
+			  forKey:PCBuilderTargets
+			  notify:NO];
+    }
+
+  return buildTargets;
 }
 
 - (NSArray *)sourceFileKeys
@@ -894,7 +974,7 @@ NSString
     }
   else if ([key isEqualToString:PCInterfaces])
     {
-      return [NSArray arrayWithObjects:@"gmodel",@"gorm", @"gsmarkup", nil];
+      return [NSArray arrayWithObjects:@"gmodel",@"gorm", @"gsmarkup", @"nib", nil];
     }
   else if ([key isEqualToString:PCImages])
     {
@@ -1143,9 +1223,9 @@ NSString
     {
       if (![fileManager copyFiles:fileList intoDirectory:directory])
 	{
-	  NSRunAlertPanel(@"Alert",
-			  @"Error adding files to project %@!",
-			  @"OK", nil, nil, projectName);
+	  NSRunAlertPanel(@"Add File(s)",
+			  @"Error adding files %@ to project %@!",
+			  @"OK", nil, nil, fileList, projectName);
 	  return NO;
 	}
 
@@ -1154,9 +1234,9 @@ NSString
       if (![fileManager copyFiles:complementaryFiles 
 	            intoDirectory:complementaryDir])
 	{
-	  NSRunAlertPanel(@"Alert",
-			  @"Error adding complementary files to project %@!",
-			  @"OK", nil, nil, projectName);
+	  NSRunAlertPanel(@"Add File(s)",
+			  @"Error adding files %@ to project %@!",
+			  @"OK", nil, nil, complementaryFiles, projectName);
 	  return NO;
 	}
     }
@@ -1459,12 +1539,14 @@ NSString
       // Subproject not found in array, load it
       if (sp == nil)
 	{
+	  NSString *pname = [NSString stringWithFormat: @"%@.pcproj",name];
 	  spFile = [projectPath stringByAppendingPathComponent:name];
 	  spFile = [spFile stringByAppendingPathExtension:@"subproj"];
-	  spFile = [spFile stringByAppendingPathComponent:@"PC.project"];
+	  spFile = [spFile stringByAppendingPathComponent: pname];
+	  spFile = [spFile stringByAppendingPathComponent: @"PC.project"];
 /*	  PCLogInfo(self, @"Not found! Load subproject: %@ at path: %@",
 		    name, spFile);*/
-	  sp = [projectManager loadProjectAt:spFile];
+	  sp = [projectManager openProjectAt:spFile makeActive:NO];
 	  if (sp)
 	    {
 	      [sp setIsSubproject:YES];
