@@ -33,6 +33,8 @@
 
 #import <ProjectCenter/PCLogController.h>
 
+#import "Modules/Preferences/Misc/PCMiscPrefs.h"
+
 NSString *PCBrowserDidSetPathNotification = @"PCBrowserDidSetPathNotification";
 
 @implementation PCProjectBrowser
@@ -150,9 +152,15 @@ NSString *PCBrowserDidSetPathNotification = @"PCBrowserDidSetPathNotification";
   NSString  *name = nil;
   int       i;
 
+  // Name of subproject selected: Change active project to superproject
+  // to check category against superproject's catgory list.
+  // But: path '/Subproject/Foo' and '/Subprojects/Foo/Subprojects' will
+  // return the same category 'Subprojects' and active project will be 'Foo'
+  // in both cases
   if ([lastPathElement isEqualToString:[activeProject projectName]])
     {
-      return [activeProject projectName];
+      activeProject = [activeProject superProject];
+      rootCategories = [activeProject rootCategories];
     }
 
   if (([rootCategories containsObject:lastPathElement]
@@ -357,30 +365,29 @@ NSString *PCBrowserDidSetPathNotification = @"PCBrowserDidSetPathNotification";
 
 - (void)click:(id)sender
 {
-  NSUserDefaults *ud;
-  NSString       *category;
-  PCProject      *activeProject;
-  NSString       *browserPath;
-  NSString       *filePath;
+  NSString  *category;
+  PCProject *activeProject;
+  NSString  *browserPath;
+  NSString  *filePath;
+  NSString  *fileName;
 
   if (sender != browser)
     {
       return;
     }
 
-  ud = [NSUserDefaults standardUserDefaults];
   category = [self nameOfSelectedCategory];
   activeProject = [[project projectManager] activeProject];
   browserPath = [self path];
   filePath = [self pathToSelectedFile];
+  fileName = [self nameOfSelectedFile];
 
-//  NSLog(@"browserPath: %@ forProject: %@", 
-//	browserPath, [activeProject projectName]);
+  NSLog(@"category: %@ forProject: %@", 
+	category, [activeProject projectName]);
 
-//  if ([[self selectedFiles] count] == 1
   if (filePath &&
       [filePath isEqualToString:browserPath] && 
-      ![[ud objectForKey:SeparateEditor] isEqualToString:@"YES"])
+      ![fileName isEqualToString:[activeProject projectName]])
     {
 //      NSLog(@"[click] category: %@ filePath: %@", category, filePath);
       [[activeProject projectEditor] openEditorForCategoryPath:browserPath
@@ -394,23 +401,22 @@ NSString *PCBrowserDidSetPathNotification = @"PCBrowserDidSetPathNotification";
 
 - (void)doubleClick:(id)sender
 {
+  NSString    *category = [self nameOfSelectedCategory];
   id          selectedCell;
-  NSString    *category;
   NSString    *fileName;
   PCProject   *activeProject;
   NSString    *key;
   NSString    *filePath;
+  id <PCPreferences> prefs = [[project projectManager] prefController];
   NSWorkspace *workspace;
   NSString    *appName, *type;
   
-  if ((sender != browser) ||
-      [[self nameOfSelectedCategory] isEqualToString:@"Libraries"])
+  if ((sender != browser) || [category isEqualToString:@"Libraries"])
     {
       return;
     }
 
   selectedCell = [sender selectedCell];
-  category = [self nameOfSelectedCategory];
   fileName = [[sender selectedCell] stringValue];
   activeProject = [[project projectManager] activeProject];
   key = [activeProject keyForCategory:category];
@@ -418,21 +424,34 @@ NSString *PCBrowserDidSetPathNotification = @"PCBrowserDidSetPathNotification";
 
   if ([self nameOfSelectedFile] != nil) 
     {
-/*      PCLogInfo(self, @"{doubleClick} filePath: %@", filePath);*/
+      BOOL foundApp = NO;
+      // PCLogInfo(self, @"{doubleClick} filePath: %@", filePath);*/
 
       workspace = [NSWorkspace sharedWorkspace];
-      [workspace getInfoForFile:filePath application:&appName type:&type];
-//      NSLog (@"Open file: %@ with app: %@", filePath, appName);
+      foundApp = [workspace getInfoForFile:filePath 
+			    application:&appName 
+				   type:&type];
+      // NSLog (@"Open file: %@ with app: %@", filePath, appName);
 
       // If 'Editor' role was set in .GNUstepExtPrefs application
       // name will be returned according that setting. Otherwise
       // 'ProjectCenter.app' will be returned accoring to NSTypes
       // from Info-gnustep.plist file of PC.
-      if (appName == nil ||
-	  [appName isEqualToString:@"ProjectCenter.app"])
+      if(foundApp == NO || [appName isEqualToString:@"ProjectCenter.app"])
 	{
-	  [[project projectEditor] openEditorForCategoryPath:[browser path]
-						    windowed:YES];
+	  appName = [prefs stringForKey:Editor];
+
+	  if (![appName isEqualToString:@"ProjectCenter"])
+	    {
+	      [workspace openFile:filePath 
+		  withApplication:appName];
+	    }
+	  else
+	    {
+	      [[activeProject projectEditor] 
+		openEditorForCategoryPath:[self path]
+				 windowed:YES];
+	    }
 	}
       else
 	{
@@ -560,11 +579,12 @@ NSString *PCBrowserDidSetPathNotification = @"PCBrowserDidSetPathNotification";
 
 - (NSImage *)fileNameIconImage
 {
-  NSString *categoryName = nil;
-  NSString *fileName = nil;
-  NSString *fileExtension = nil;
-  NSString *iconName = nil;
-  NSImage  *icon = nil;
+  NSString  *categoryName = nil;
+  NSString  *fileName = nil;
+  NSString  *fileExtension = nil;
+  NSString  *iconName = nil;
+  NSImage   *icon = nil;
+  PCProject *activeProject = [[project projectManager] activeProject];
 
   fileName = [self nameOfSelectedFile];
   if (fileName)
@@ -587,7 +607,9 @@ NSString *PCBrowserDidSetPathNotification = @"PCBrowserDidSetPathNotification";
     {
       iconName = [[NSString alloc] initWithString:@"MultiFiles"];
     }
-  else if (!categoryName && !fileName) // Nothing selected
+  // Nothing or subproject name selected
+  else if ((!categoryName && !fileName) ||
+	   [fileName isEqualToString:[activeProject projectName]]) 
     {
       iconName = [[NSString alloc] initWithString:@"FileProject"];
     }
@@ -671,6 +693,15 @@ NSString *PCBrowserDidSetPathNotification = @"PCBrowserDidSetPathNotification";
   return PCFileNameFieldNoFiles;
 }
 
+- (NSString *)fileNameIconPath
+{
+  NSString *fileName = [self nameOfSelectedFile];
+  NSString *category = [self nameOfSelectedCategory];
+
+  return [project pathForFile:fileName 
+		       forKey:[project keyForCategory:category]];
+}
+
 - (BOOL)canPerformDraggingOf:(NSArray *)paths
 {
   NSString     *category = [self nameOfSelectedCategory];
@@ -681,7 +712,7 @@ NSString *PCBrowserDidSetPathNotification = @"PCBrowserDidSetPathNotification";
 
   NSLog(@"PCBrowser: canPerformDraggingOf -> %@", category);
 
-  if (!category)
+  if (!category || ([self nameOfSelectedFile] != nil))
     {
       return NO;
     }
@@ -691,6 +722,7 @@ NSString *PCBrowserDidSetPathNotification = @"PCBrowserDidSetPathNotification";
       return NO;
     }
 
+  // Check if we can accept files of such types
   while ((s = [e nextObject]))
     {
       if (![fileTypes containsObject:[s pathExtension]])
@@ -702,4 +734,27 @@ NSString *PCBrowserDidSetPathNotification = @"PCBrowserDidSetPathNotification";
   return YES;
 }
 
+- (BOOL)prepareForDraggingOf:(NSArray *)paths
+{
+  return YES;
+}
+
+- (BOOL)performDraggingOf:(NSArray *)paths
+{
+  NSString     *category = [self nameOfSelectedCategory];
+  NSString     *categoryKey = [project keyForCategory:category];
+  NSEnumerator *pathsEnum = [paths objectEnumerator];
+  NSString     *file = nil;
+
+  while ((file = [[pathsEnum nextObject] lastPathComponent]))
+    {
+      if (![project doesAcceptFile:file forKey:categoryKey])
+	{
+	  return NO;
+	}
+    }
+
+  return [project addAndCopyFiles:paths forKey:categoryKey];
+}
+
 @end
