@@ -23,24 +23,27 @@
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
 */
 
-#include <AppKit/AppKit.h>
+#import <AppKit/AppKit.h>
 
-#include <ProjectCenter/PCDefines.h>
-#include <ProjectCenter/PCSplitView.h>
-#include <ProjectCenter/PCButton.h>
+#import <ProjectCenter/PCDefines.h>
+#import <ProjectCenter/PCButton.h>
 
-#include <ProjectCenter/PCFileManager.h>
+#import <ProjectCenter/PCFileManager.h>
 
-#include <ProjectCenter/PCProjectManager.h>
-#include <ProjectCenter/PCProject.h>
-#include <ProjectCenter/PCProjectBuilder.h>
-#include <ProjectCenter/PCProjectBuilderOptions.h>
+#import <ProjectCenter/PCProjectManager.h>
+#import <ProjectCenter/PCProject.h>
+#import <ProjectCenter/PCProjectWindow.h>
+#import <ProjectCenter/PCProjectBuilder.h>
+#import <ProjectCenter/PCProjectBuilderOptions.h>
 
-#include <ProjectCenter/PCProjectEditor.h>
-#include <Protocols/CodeEditor.h>
+#import <ProjectCenter/PCProjectEditor.h>
+#import <Protocols/CodeEditor.h>
+#import <ProjectCenter/PCSaveModified.h>
 
-#include <ProjectCenter/PCLogController.h>
-#include <ProjectCenter/PCPrefController.h>
+#import <ProjectCenter/PCLogController.h>
+#import <Protocols/Preferences.h>
+
+#import "../Modules/Preferences/Build/PCBuildPrefs.h"
 
 #ifndef IMAGE
 #define IMAGE(X) [NSImage imageNamed: X]
@@ -54,9 +57,11 @@
 
 - (id)initWithProject:(PCProject *)aProject
 {
-  NSAssert(aProject, @"No project specified!");
+#ifdef DEBUG
+  NSLog (@"PCProjectBuilder: initWithProject");
+#endif
 
-//  PCLogInfo(self, @"initWithProject %@", [aProject projectName]);
+  NSAssert(aProject, @"No project specified!");
   
   if ((self = [super init]))
     {
@@ -77,6 +82,12 @@
 	  PCLogError(self, @"error loading Builder NIB file!");
 	  return nil;
 	}
+      [[NSNotificationCenter defaultCenter]
+	addObserver:self
+	   selector:@selector(loadPreferences:)
+	       name:PCPreferencesDidChangeNotification
+	     object:nil];
+      [self loadPreferences:nil];
     }
 
   return self;
@@ -84,21 +95,35 @@
 
 - (void)dealloc
 {
-#ifdef DEVELOPMENT
+#ifdef DEBUG
   NSLog (@"PCProjectBuilder: dealloc");
 #endif
 
-  [buildStatusTarget release];
-  [buildTarget release];
-  [buildArgs release];
-  [makePath release];
+  [[NSNotificationCenter defaultCenter] removeObserver:self];
 
-//  PCLogInfo(self, @"componentView RC: %i", [componentView retainCount]);
-//  PCLogInfo(self, @"RC: %i", [self retainCount]);
-  [componentView release];
-  [errorArray release];
-  [errorString release];
-  [buildOptions release];
+  if ([componentView superview])
+    {
+      [componentView removeFromSuperview];
+    }
+
+  RELEASE(buildStatusTarget);
+  RELEASE(buildTarget);
+  RELEASE(buildArgs);
+  RELEASE(buildOptions);
+
+  RELEASE(successSound);
+  RELEASE(failureSound);
+  RELEASE(rootBuildDir);
+  RELEASE(buildTool);
+
+//  NSLog(@"Project Builder--> componentView RC: %i", 
+//	[componentView retainCount]);
+
+  RELEASE(componentView);
+  RELEASE(errorArray);
+  RELEASE(errorString);
+
+//  NSLog(@"Project Builder--> RC: %i", [self retainCount]);
 
   [super dealloc];
 }
@@ -113,11 +138,14 @@
       return;
     }
 
-  NSLog(@"ProjectBuilder awakeFromNib");
+//  NSLog(@"ProjectBuilder awakeFromNib");
 
   [componentView retain];
   [componentView removeFromSuperview];
 
+//  NSLog(@"ProjectBuilder awakeFromNib: componentView RC:%i", 
+//	[componentView retainCount]);
+
   /*
    * 4 build Buttons
    */
@@ -159,9 +187,9 @@
   [errorOutput addTableColumn:errorColumn];
   [errorOutput setDataSource:self];
   [errorOutput setBackgroundColor:[NSColor colorWithDeviceRed:0.88
-                                                             green:0.76 
-                                                              blue:0.60 
-                                                             alpha:1.0]];
+			                                green:0.76 
+			                                 blue:0.60 
+			                                alpha:1.0]];
   [errorOutput setDrawsGrid:NO];
   [errorOutput setTarget:self];
   [errorOutput setAction:@selector(errorItemClick:)];
@@ -178,8 +206,7 @@
   /*
    *  Log output
    */
-  logScroll = [[NSScrollView alloc] 
-    initWithFrame:NSMakeRect (0, 0, 480, 133)];
+  logScroll = [[NSScrollView alloc] initWithFrame:NSMakeRect(0,0,480,133)];
   [logScroll setHasHorizontalScroller:NO];
   [logScroll setHasVerticalScroller:YES];
   [logScroll setBorderType:NSBezelBorder];
@@ -209,9 +236,9 @@
    * Split view
    */
   [split addSubview:errorScroll];
-  RELEASE (errorScroll);
+  RELEASE(errorScroll);
   [split addSubview:logScroll];
-  RELEASE (logScroll);
+  RELEASE(logScroll);
 
 //  [split adjustSubviews];
 //  [componentView addSubview:split];
@@ -225,19 +252,17 @@
   return componentView;
 }
 
-- (BOOL)setMakePath
+- (void)loadPreferences:(NSNotification *)aNotification
 {
-  makePath = [[NSUserDefaults standardUserDefaults] objectForKey:BuildTool];
+  id <PCPreferences> prefs = [[project projectManager] prefController];
 
-  if (!makePath || ![[NSFileManager defaultManager] fileExistsAtPath:makePath])
-    {
-      NSRunAlertPanel(@"Build terminated",
-  		      @"Build tool not found.\nFile \"%@\" doesn't exist!",
-  		      @"OK", nil, nil, makePath);
-      return NO;
-    }
+  ASSIGN(successSound, [prefs stringForKey:SuccessSound]);
+  ASSIGN(failureSound, [prefs stringForKey:FailureSound]);
 
-  return YES;
+  ASSIGN(rootBuildDir, [prefs stringForKey:RootBuildDirectory]);
+  ASSIGN(buildTool, [prefs stringForKey:BuildTool]);
+
+  promptOnClean = [prefs boolForKey:PromptOnClean];
 }
 
 - (void)updateTargetField
@@ -248,6 +273,8 @@
   args = [[[project projectDict] objectForKey:PCBuilderArguments] 
     componentsJoinedByString:@" "];
 
+  if (!args) args = @" ";
+
   s = [NSString stringWithFormat:@"%@ with args '%@'", buildTarget, args];
 
   [targetField setStringValue:s];
@@ -302,18 +329,15 @@
       ![instDir isEqualToString:@"SYSTEM"] &&
       ![instDir isEqualToString:@"USER"] &&
       ![instDir isEqualToString:@"NETWORK"] &&
-      ![instDir isEqualToString:@""])
+      ![instDir isEqualToString:@""] &&
+      ([instDir isAbsolutePath] || [instDir characterAtIndex:0] == '$'))
     {
       [args addObject:[NSString stringWithFormat:@"DESTDIR=%@", instDir]];
     }
 
   [args addObjectsFromArray:[projectDict objectForKey:PCBuilderArguments]];
 
-  // Get arguments from options
-  if ([[projectDict objectForKey:PCBuilderVerbose] isEqualToString:@"YES"])
-    { // default is 'messages=no'
-      [args addObject:@"messages=yes"];
-    }
+  // --- Get arguments from options
   if ([[projectDict objectForKey:PCBuilderDebug] isEqualToString:@"NO"])
     { // default is 'debug=yes'
       [args addObject:@"debug=no"];
@@ -326,6 +350,18 @@
     { // default is 'shared=yes'
       [args addObject:@"shared=no"];
     }
+  // Always add 'messages=yes' argument. Build output parsing assumes this.
+  [args addObject:@"messages=yes"];
+  // "Verbose ouput" option (Build Options panel) just toogle if build shows
+  // as with argument 'messages=yes' or not.
+  if ([[projectDict objectForKey:PCBuilderVerbose] isEqualToString:@"YES"])
+    {
+      verboseBuilding = YES;
+    }
+  else
+    {
+      verboseBuilding = NO;
+    }
 
   return args;
 }
@@ -338,27 +374,22 @@
       return;
     }
 
-  [buildArgs addObject:buildTarget];
-
   // Set build arguments
+  [buildArgs addObject:buildTarget];
   [buildArgs addObjectsFromArray:[self buildArguments]];
  
-  NSLog(@"ProjectBuilder arguments: %@", buildArgs);
+//  NSLog(@"ProjectBuilder arguments: %@", buildArgs);
 
   currentEL = ELNone;
   lastEL = ELNone;
   nextEL = ELNone;
   lastIndentString = @"";
 
-  currentBuildPath = [[NSMutableArray alloc] initWithCapacity:1];
-  [currentBuildPath addObject:[project projectPath]];
-  currentBuildFile = [[NSMutableString alloc] initWithString:@""];
-
   buildStatus = [NSString stringWithString:@"Building..."];
   [buildStatusTarget setString:@"Build"];
   [cleanButton setEnabled:NO];
-  [self build:self];
   _isBuilding = YES;
+  [self build:self];
 }
 
 - (void)startClean:(id)sender
@@ -368,12 +399,11 @@
       return;
     }
 
-  if ([[[[NSUserDefaults standardUserDefaults] dictionaryRepresentation]
-      objectForKey:PromptOnClean] isEqualToString:@"YES"])
+  if (promptOnClean)
     {
-      if (NSRunAlertPanel(@"Clean Project?",
+      if (NSRunAlertPanel(@"Project Clean",
 			  @"Do you really want to clean project '%@'?",
-			  @"Yes", @"No", nil, [project projectName])
+			  @"Clean", @"Stop", nil, [project projectName])
 	  == NSAlertAlternateReturn)
 	{
 	  [cleanButton setState:NSOffState];
@@ -381,19 +411,20 @@
 	}
     }
 
+  // Set build arguments
+  [buildArgs addObject:@"clean"];
+  [buildArgs addObjectsFromArray:[self buildArguments]];
+
   buildStatus = [NSString stringWithString:@"Cleaning..."];
   [buildStatusTarget setString:@"Clean"];
-  [buildArgs addObject:@"clean"];
   [buildButton setEnabled:NO];
-  [self build:self];
   _isCleaning = YES;
+  [self build:self];
 }
 
 - (BOOL)stopMake:(id)sender
 {
-  // [makeTask isRunning] doesn't work here.
-  // "waitpid 7045, result -1, error No child processes" is printed.
-  if (makeTask)
+  if (makeTask && [makeTask isRunning])
     {
       PCLogStatus(self, @"task will terminate");
       NS_DURING
@@ -418,81 +449,105 @@
 
 - (void)cleanupAfterMake
 {
+  NSString *statusString;
+
   if (_isBuilding || _isCleaning)
     {
-      [statusField setStringValue:[NSString stringWithFormat: 
-	@"%@ - %@ terminated", [project projectName], buildStatusTarget]];
+      statusString =[NSString stringWithFormat: 
+	@"%@ - %@ terminated", [project projectName], buildStatusTarget];
+      [statusField setStringValue:statusString];
+      [[project projectWindow] updateStatusLineWithText:statusString];
     }
 
   // Restore buttons state
-  if ([buildStatusTarget isEqualToString:@"Build"])
+  if (_isBuilding)
     {
       [buildButton setState:NSOffState];
       [cleanButton setEnabled:YES];
+      _isBuilding = NO;
     }
-  else if ([buildStatusTarget isEqualToString:@"Clean"])
+  else if (_isCleaning)
     {
       [cleanButton setState:NSOffState];
       [buildButton setEnabled:YES];
+      _isCleaning = NO;
     }
 
   [buildArgs removeAllObjects];
   [buildStatusTarget setString:@"Default"];
 
-  if (_isBuilding)
-    {
-      [currentBuildPath release];
-      [currentBuildFile release];
-    }
-
-  _isBuilding = NO;
-  _isCleaning = NO;
+  // Initiated in [self build:]
+  [currentBuildPath release];
+  [currentBuildFile release];
 }
 
 // --- Actions
 - (BOOL)prebuildCheck
 {
-  PCPrefController *prefs = [PCPrefController sharedPCPreferences];
-  PCFileManager    *pcfm = [PCFileManager defaultManager];
-  NSFileManager    *fm = [NSFileManager defaultManager];
-  NSString         *buildDir = [prefs objectForKey:RootBuildDirectory];
-  NSString         *projectBuildDir;
+  PCFileManager   *pcfm = [PCFileManager defaultManager];
+  NSFileManager   *fm = [NSFileManager defaultManager];
+  NSString        *buildDir;
+  PCProjectEditor *projectEditor;
+  int             ret;
 
-  // Checking prerequisites
+  // Checking for project 'edited' state 
   if ([project isProjectChanged])
     {
-      if (NSRunAlertPanel(@"Project Changed!",
-			  @"Should it be saved first?",
-			  @"Yes", @"No", nil) == NSAlertDefaultReturn) 
+      ret = NSRunAlertPanel(@"Project Build",
+  			    @"Project was changed and not saved.\n"
+  			    @"Do you want to save project before building it?",
+  			    @"Stop Build", @"Save and Build", nil);
+      switch (ret)
 	{
+	case NSAlertDefaultReturn: // Stop Build
+	  return NO;
+	  break;
+
+	case NSAlertAlternateReturn: // Save Project
 	  [project save];
+	  break;
 	}
     }
   else
     {
-      // Synchronize PC.project and generated files just for case
+      // Synchronize PC.project and generate files
       [project save];
     }
 
-  // Get make tool path
-  makePath = [[NSUserDefaults standardUserDefaults] objectForKey:BuildTool];
-
-  if (!makePath || ![[NSFileManager defaultManager] fileExistsAtPath:makePath])
+  // Checking if edited files exist
+  projectEditor = [project projectEditor];
+  if ([projectEditor hasModifiedFiles])
     {
-      NSRunAlertPanel(@"Build terminated",
-  		      @"Build tool not found.\nFile \"%@\" doesn't exist!",
-  		      @"OK", nil, nil, makePath);
+      if (!PCRunSaveModifiedFilesPanel(projectEditor, 
+				       @"Save and Build",
+				       @"Build Anyway",
+				       @"Cancel"))
+	{
+	  return NO;
+	}
+    }
+
+  // Check build tool path
+  if (!buildTool || ![fm fileExistsAtPath:buildTool])
+    {
+      NSRunAlertPanel(@"Project Build",
+  		      @"Build tool '%@' not found. Check preferences.\n"
+		      @"Build will be terminated.",
+  		      @"Close", nil, nil, buildTool);
       return NO;
     }
 
   // Create root build directory if not exist
-  projectBuildDir = [NSString stringWithFormat:@"%@.build", 
-		  [project projectName]];
-  projectBuildDir = [buildDir stringByAppendingPathComponent:projectBuildDir];
-  if (![fm fileExistsAtPath:buildDir] ||
-      ![fm fileExistsAtPath:projectBuildDir])
+  if (rootBuildDir && ![rootBuildDir isEqualToString:@""])
     {
-      [pcfm createDirectoriesIfNeededAtPath:projectBuildDir];
+      buildDir = [NSString 
+	stringWithFormat:@"%@.build", [project projectName]];
+      buildDir = [rootBuildDir stringByAppendingPathComponent:buildDir];
+      if (![fm fileExistsAtPath:rootBuildDir] || 
+	  ![fm fileExistsAtPath:buildDir])
+	{
+	  [pcfm createDirectoriesIfNeededAtPath:buildDir];
+	}
     }
 
   return YES;
@@ -500,11 +555,11 @@
 
 - (void)build:(id)sender
 {
-  NSPipe *logPipe;
-  NSPipe *errorPipe;
-
-  // TODO: Support build options!!!
-  //  NSDictionary        *optionDict = [project buildOptions];
+  // Make runtime vars
+  // Released in [self cleanupAfterMake]
+  currentBuildPath = [[NSMutableString alloc] 
+    initWithString:[project projectPath]];
+  currentBuildFile = [[NSMutableString alloc] initWithString:@""];
 
   // Checking build conditions
   if ([self prebuildCheck] == NO)
@@ -514,30 +569,32 @@
     }
 
   // Prepearing to building
-  logPipe = [NSPipe pipe];
-  readHandle = [logPipe fileHandleForReading];
-  [readHandle waitForDataInBackgroundAndNotify];
+  _isLogging = YES;
+  stdOutPipe = [[NSPipe alloc] init];
+  stdOutHandle = [stdOutPipe fileHandleForReading];
+  [stdOutHandle waitForDataInBackgroundAndNotify];
 
   [NOTIFICATION_CENTER addObserver:self 
                           selector:@selector(logStdOut:)
 			      name:NSFileHandleDataAvailableNotification
-			    object:readHandle];
-  _isLogging = YES;
+			    object:stdOutHandle];
 
-  errorPipe = [NSPipe pipe];
-  errorReadHandle = [errorPipe fileHandleForReading];
-  [errorReadHandle waitForDataInBackgroundAndNotify];
+  _isErrorLogging = YES;
+  stdErrorPipe = [[NSPipe alloc] init];
+  stdErrorHandle = [stdErrorPipe fileHandleForReading];
+  [stdErrorHandle waitForDataInBackgroundAndNotify];
 
   [NOTIFICATION_CENTER addObserver:self 
                           selector:@selector(logErrOut:) 
 			      name:NSFileHandleDataAvailableNotification
-			    object:errorReadHandle];
-  _isErrorLogging = YES;
+			    object:stdErrorHandle];
+
   [errorsCountField setStringValue:[NSString stringWithString:@""]];
   errorsCount = 0;
   warningsCount = 0;
 
   [statusField setStringValue:buildStatus];
+  [[project projectWindow] updateStatusLineWithText:buildStatus];
 
   // Run make task
   [logOutput setString:@""];
@@ -552,9 +609,13 @@
   makeTask = [[NSTask alloc] init];
   [makeTask setArguments:buildArgs];
   [makeTask setCurrentDirectoryPath:[project projectPath]];
-  [makeTask setLaunchPath:makePath];
-  [makeTask setStandardOutput:logPipe];
-  [makeTask setStandardError:errorPipe];
+  [makeTask setLaunchPath:buildTool];
+  [makeTask setStandardOutput:stdOutPipe];
+  [makeTask setStandardError:stdErrorPipe];
+
+  [self logBuildString:
+    [NSString stringWithFormat:@"=== %@ started ===", buildStatusTarget]
+	       newLine:YES];
 
   NS_DURING
     {
@@ -576,14 +637,16 @@
 
 - (void)buildDidTerminate:(NSNotification *)aNotif
 {
-  int status;
+  int      status;
+  NSString *logString;
+  NSString *statusString;
 
   if ([aNotif object] != makeTask)
     {
       return;
     }
 
-  NSLog(@"task did terminate");
+//  NSLog(@"task did terminate");
 
   [NOTIFICATION_CENTER removeObserver:self 
 			         name:NSTaskDidTerminateNotification
@@ -601,46 +664,49 @@
   NS_ENDHANDLER
  
   // Finish task
+  // TODO: Strange behaviour of pipe and file handlers alloc/release. Also
+  // they have big retain count here (2 or 3). Why? Notification retains it?
   RELEASE(makeTask);
   makeTask = nil;
 
-  // Wait for logging end
+  // Wait while logging ends
   while (_isLogging || _isErrorLogging) 
     {
       [[NSRunLoop currentRunLoop] runMode:NSDefaultRunLoopMode
 			       beforeDate:[NSDate distantFuture]];
     }
-    
+
+  RELEASE(stdOutPipe);
+  RELEASE(stdErrorPipe);
+
   [self updateErrorsCountField];
 
   if (status == 0)
     {
-      [self logString: 
-	[NSString stringWithFormat:@"=== %@ succeeded! ===", buildStatusTarget] 
-	                     error:NO
-			   newLine:YES];
-      [statusField setStringValue:[NSString stringWithFormat: 
-	@"%@ - %@ succeeded", [project projectName], buildStatusTarget]];
+      logString = [NSString stringWithFormat:@"=== %@ succeeded! ===", 
+		buildStatusTarget];
+      statusString = [NSString stringWithFormat:@"%@ - %@ succeeded", 
+		   [project projectName], buildStatusTarget];
     } 
   else
     {
-      [self logString: 
-	[NSString stringWithFormat:@"=== %@ terminated! ===", buildStatusTarget]
-	                     error:NO
-			   newLine:YES];
+      logString = [NSString stringWithFormat:@"=== %@ terminated! ===", 
+		buildStatusTarget];
       if (errorsCount > 0)
 	{
-	  [statusField setStringValue:[NSString stringWithFormat: 
+	  statusString = [NSString stringWithFormat: 
 	    @"%@ - %@ failed (%i errors)", 
-	    [project projectName], buildStatusTarget, errorsCount]];
+	    [project projectName], buildStatusTarget, errorsCount];
 	}
       else
 	{
-	  [statusField setStringValue:[NSString stringWithFormat: 
-	    @"%@ - %@ failed", 
-	    [project projectName], buildStatusTarget]];
+	  statusString = [NSString stringWithFormat:@"%@ - %@ failed", 
+		       [project projectName], buildStatusTarget];
 	}
     }
+  [statusField setStringValue:statusString];
+  [[project projectWindow] updateStatusLineWithText:statusString];
+  [self logBuildString:logString newLine:YES];
 
   // Run post process if configured
 /*  if (status && postProcess)
@@ -649,63 +715,24 @@
       postProcess = NULL;
     }*/
 
-  _isBuilding = NO;
-  _isCleaning = NO;
   [self cleanupAfterMake];
 }
 
-- (void)logStdOut:(NSNotification *)aNotif
+// --- BuilderOptions delegate
+- (void)targetDidSet:(NSString *)target
 {
-  NSData *data;
-
-//  NSLog(@"logStdOut");
-
-  if ((data = [readHandle availableData]) && [data length] > 0)
-    {
-      [self logData:data error:NO];
-    }
-
-  if (makeTask)
-    {
-      [readHandle waitForDataInBackgroundAndNotify];
-    }
-  else
-    {
-      _isLogging = NO;
-      [NOTIFICATION_CENTER removeObserver:self 
-			             name:NSFileHandleDataAvailableNotification
-			           object:readHandle];
-    }
+  [buildTarget setString:target];
+  [self updateTargetField];
 }
 
-- (void)logErrOut:(NSNotification *)aNotif
-{
-  NSData *data;
+@end
 
-//  NSLog(@"logErrOut");
-  
-  if ((data = [errorReadHandle availableData]) && [data length] > 0)
-    {
-      [self logData:data error:YES];
-    }
-
-  if (makeTask)
-    {
-      [errorReadHandle waitForDataInBackgroundAndNotify];
-    }
-  else
-    {
-      _isErrorLogging = NO;
-      [NOTIFICATION_CENTER removeObserver:self 
-			             name:NSFileHandleDataAvailableNotification
-			           object:errorReadHandle];
-    }
-}
+@implementation PCProjectBuilder (Logging)
 
 - (void)updateErrorsCountField
 {
   NSString *string;
-  NSString *errorsString = [NSString stringWithString:@""];;
+  NSString *errorsString = [NSString stringWithString:@""];
   NSString *warningsString = [NSString stringWithString:@""];
 
   if (errorsCount > 0)
@@ -738,44 +765,55 @@
   [errorsCountField setStringValue:string];
 }
 
-// --- BuilderOptions delgate
-- (void)targetDidSet:(NSString *)target
+// --- Data notifications
+// Both methods make call to dipatcher logData:error:
+- (void)logStdOut:(NSNotification *)aNotif
 {
-  [buildTarget setString:target];
-  [self updateTargetField];
-}
+  NSData *data;
 
-@end
-
-@implementation PCProjectBuilder (BuildLogging)
-
-- (void)logString:(NSString *)str
-            error:(BOOL)yn
-	  newLine:(BOOL)newLine
-{
-//  NSTextView *out = (yn) ? errorOutput : logOutput;
-  NSTextView *out = logOutput;
-
-  [out replaceCharactersInRange:
-    NSMakeRange([[out string] length],0) withString:str];
-
-  if (newLine)
+  if ((data = [stdOutHandle availableData]) && [data length] > 0)
     {
-      [out replaceCharactersInRange:
-	NSMakeRange([[out string] length], 0) withString:@"\n"];
+      [self logData:data error:NO];
+    }
+
+  if (makeTask)
+    {
+      [stdOutHandle waitForDataInBackgroundAndNotify];
     }
   else
     {
-      [out replaceCharactersInRange:
-	NSMakeRange([[out string] length], 0) withString:@" "];
+      [NOTIFICATION_CENTER removeObserver:self 
+			             name:NSFileHandleDataAvailableNotification
+			           object:stdOutHandle];
+      _isLogging = NO;
     }
-
-  [out scrollRangeToVisible:NSMakeRange([[out string] length], 0)];
-  [out setNeedsDisplay:YES];
 }
 
+- (void)logErrOut:(NSNotification *)aNotif
+{
+  NSData *data;
+
+  if ((data = [stdErrorHandle availableData]) && [data length] > 0)
+    {
+      [self logData:data error:YES];
+    }
+
+  if (makeTask)
+    {
+      [stdErrorHandle waitForDataInBackgroundAndNotify];
+    }
+  else
+    {
+      [NOTIFICATION_CENTER removeObserver:self 
+			             name:NSFileHandleDataAvailableNotification
+			           object:stdErrorHandle];
+      _isErrorLogging = NO;
+    }
+}
+
+// --- Dispatching
 - (void)logData:(NSData *)data
-          error:(BOOL)yn
+          error:(BOOL)isError
 {
   NSString *dataString;
   NSRange  newLineRange;
@@ -788,6 +826,9 @@
 			    
   // Process new data
   lineRange.location = 0;
+  newLineRange.location = 0;
+  // 'errorString' collects data across logData:error: calls until 
+  // new line character is appeared.
   [errorString appendString:dataString];
   while (newLineRange.location != NSNotFound)
     {
@@ -805,16 +846,18 @@
 	  lineString = [errorString substringWithRange:lineRange];
 	  [errorString deleteCharactersInRange:lineRange];
 
-	  // Send it
-	  if (_isBuilding)
+	  // Send it to error view
+	  // Do not process make errors in other mode but building. Maybe
+	  // some day...
+	  if (_isBuilding && isError)
 	    {
-	      [self parseBuildLine:lineString];
-	      if (yn)
-		{
-		  [self logErrorString:lineString];
-		}
+    	      [self logErrorString:lineString];
+	    }
+	  // Cleaning or building standard out string
+	  if (!isError || verboseBuilding)
+	    {
+	      [self logBuildString:lineString newLine:NO];
 	    }
-	  [self logString:lineString error:yn newLine:NO];
 	}
       else
 	{
@@ -826,39 +869,180 @@
   RELEASE(dataString);
 }
 
-- (void)parseBuildLine:(NSString *)string
-{
-  NSArray  *components = [string componentsSeparatedByString:@" "];
+@end
 
-  if (!components)
+@implementation PCProjectBuilder (BuildLogging)
+
+// --- Parsing utilities
+- (BOOL)line:(NSString *)lineString startsWithString:(NSString *)substring
+{
+  int      position = 0;
+  NSRange  range = NSMakeRange(position,1);
+
+  while ([[lineString substringFromRange:range] isEqualToString:@" "])
+    {
+      range.location = ++position;
+    }
+
+/*  NSLog(@"Line '%@' position: %i substring '%@'", 
+	lineString, position, substring);*/
+
+  range = [lineString rangeOfString:substring];
+  if ((range.location == NSNotFound) ||
+      (range.location != position))
+    {
+      return NO;
+    }
+
+  return YES;
+}
+
+// Clean leading spaces and return cleaned array of components
+- (NSArray *)componentsOfLine:(NSString *)lineString
+{
+  NSArray        *lineComponents;
+  NSMutableArray *tempComponents;
+
+  lineComponents = [lineString componentsSeparatedByString:@" "];
+  tempComponents = [NSMutableArray arrayWithArray:lineComponents];
+
+  while ([[tempComponents objectAtIndex:0] isEqualToString:@""])
+    {
+      [tempComponents removeObjectAtIndex:0];
+    }
+
+  return tempComponents;
+}
+
+// Line starts with 'gmake' or 'make'.
+// Changes 'currentBuildPath' if line starts with 
+// "Entering directory" or "Leaving directory".
+// For example: 
+// gmake[1]: Entering directory '/Users/me/Project/Subproject.subproj'
+- (void)parseMakeLine:(NSString *)lineString
+{
+  NSMutableArray *makeLineComponents;
+  NSString       *makeLine;
+  NSString       *pathComponent;
+  NSString       *path;
+
+//  NSLog(@"parseMakeLine: %@", lineString);
+
+  makeLineComponents = [NSMutableArray 
+    arrayWithArray:[lineString componentsSeparatedByString:@" "]];
+
+  // Don't check for item at index 0 contents (it's 'gmake[1]:' or 'make[1]:') 
+  // just remove it.
+  [makeLineComponents removeObjectAtIndex:0];
+  makeLine = [makeLineComponents componentsJoinedByString:@" "];
+
+  if ([self line:makeLine startsWithString:@"Entering directory"])
+    {
+      pathComponent = [makeLineComponents objectAtIndex:2];
+      path = [pathComponent
+	substringWithRange:NSMakeRange(1,[pathComponent length]-3)]; 
+//      NSLog(@"Go down to %@", path);
+      [currentBuildPath setString:path];
+    }
+  else if ([self line:makeLine startsWithString:@"Leaving directory"])
+    {
+//      NSLog(@"Go up from %@", [makeLineComponents objectAtIndex:2]);
+      [currentBuildPath 
+	setString:[currentBuildPath stringByDeletingLastPathComponent]];
+    }
+//  NSLog(@"Current build path: %@", currentBuildPath);
+}
+
+// Should return:
+// 'Compiling ...'
+// 'Linking ...'
+// Also updates currentBuildFile
+- (NSString *)parseCompilerLine:(NSString *)lineString
+{
+  NSArray  *lineComponents = [self componentsOfLine:lineString];
+  NSString *outputString = nil;
+
+  if ([lineComponents containsObject:@"-c"])
+    {
+      [currentBuildFile setString:[lineComponents objectAtIndex:1]];
+      outputString = [NSString 
+	stringWithFormat:@" Compiling %@...\n", currentBuildFile];
+    }
+  else if ([lineComponents containsObject:@"-rdynamic"])
+    {
+      outputString = [NSString 
+	stringWithFormat:@" Linking %@...\n", 
+	[lineComponents objectAtIndex:[lineComponents indexOfObject:@"-o"]+1]];
+    }
+
+  return outputString;
+}
+// --- Parsing utilities end
+
+// Log output
+- (void)logBuildString:(NSString *)string
+	       newLine:(BOOL)newLine
+{
+  NSString *logString = [self parseBuildLine:string];
+
+  if (!logString)
     {
       return;
     }
 
-  if ([components containsObject:@"Compiling"] &&
-      [components containsObject:@"file"])
-    {
-      NSLog(@"Current build file: %@", [components objectAtIndex:3]);
-      [currentBuildFile setString:[components objectAtIndex:3]];
-    }
-  else if ([components containsObject:@"Entering"] &&
-	   [components containsObject:@"directory"])
-    {
-      NSString *path;
-      NSString *pathComponent = [components objectAtIndex:3];
+  [logOutput replaceCharactersInRange:
+    NSMakeRange([[logOutput string] length],0) withString:logString];
 
-      NSLog(@"Go down to %@", pathComponent);
-      path = [pathComponent
-	substringWithRange:NSMakeRange(1,[pathComponent length]-3)]; 
-      [currentBuildPath addObject:path];
-      NSLog(@"%@", [currentBuildPath lastObject]);
-    }
-  else if ([components containsObject:@"Leaving"] &&
-	   [components containsObject:@"directory"])
+  if (newLine)
     {
-      NSLog(@"Go up from %@", [components objectAtIndex:3]);
-      [currentBuildPath removeLastObject];
-      NSLog(@"%@", [currentBuildPath lastObject]);
+      [logOutput replaceCharactersInRange:
+	NSMakeRange([[logOutput string] length], 0) withString:@"\n"];
+    }
+
+  [logOutput scrollRangeToVisible:NSMakeRange([[logOutput string] length], 0)];
+  [logOutput setNeedsDisplay:YES];
+}
+
+// Standard out is parsed for detection of directory, file, etc.
+// Gets complete line (ended with '\n') as argument
+- (NSString *)parseBuildLine:(NSString *)string
+{
+  NSArray  *components = [self componentsOfLine:string];
+  NSString *parsedString = nil;
+
+  if (!components)
+    {
+      return nil;
+    }
+
+  if ([self line:string startsWithString:@"gmake"] ||
+      [self line:string startsWithString:@"make"])
+    {// Do current path detection
+      [self parseMakeLine:string];
+    }
+  else if ([self line:string startsWithString:@"gcc"])
+    {// Parse compiler output
+      parsedString = [self parseCompilerLine:string];
+    }
+  else if ([self line:string startsWithString:@"Making"] ||
+	   [self line:string startsWithString:@"==="])
+    {// It's a gnustep-make and self output
+      parsedString = string;
+    }
+
+  if (parsedString && ![self line:parsedString startsWithString:@"==="])
+    {
+      [statusField setStringValue:parsedString];
+      [[project projectWindow] updateStatusLineWithText:parsedString];
+    }
+
+  if (verboseBuilding)
+    {
+      return string;
+    }
+  else
+    {
+      return parsedString;
     }
 }
 
@@ -866,6 +1050,7 @@
 
 @implementation PCProjectBuilder (ErrorLogging)
 
+// Entry point for error logging
 - (void)logErrorString:(NSString *)string
 {
   NSArray *items;
@@ -879,7 +1064,7 @@
     }
 }
 
-
+// Used for warning or error message retrieval
 - (NSString *)lineTail:(NSString*)line afterString:(NSString*)string
 {
   NSRange substrRange;
@@ -952,14 +1137,15 @@
       NSString *substr;
 
       // file and includedFile
-      file = [[currentBuildPath lastObject] 
+      file = [currentBuildPath  
 	stringByAppendingPathComponent:currentBuildFile];
       if (lastEL == ELIncluded 
 	  || [[components objectAtIndex:0] isEqualToString:lastIncludedFile])
 	{// first message after "In file included from"
 //	  NSLog(@"Inlcuded File: %@", file);
 	  includedFile = [components objectAtIndex:0];
-      	  file = includedFile;
+      	  file = 
+	    [currentBuildPath stringByAppendingPathComponent:includedFile];
 	  currentEL = ELIncludedError;
 	}
       else
@@ -1088,6 +1274,7 @@
   return items;
 }
 
+// --- Error output table delegate methods
 - (int)numberOfRowsInTableView:(NSTableView *)aTableView
 {
   if (errorArray != nil && aTableView == errorOutput)
@@ -1128,6 +1315,7 @@
   if (editor)
     {
       position = NSPointFromString([error objectForKey:@"Position"]);
+      [projectEditor orderFrontEditorForFile:[error objectForKey:@"File"]];
       [editor scrollToLineNumber:(unsigned int)position.x];
 
 /*      NSLog(@"%i: %@(%@): %@", 
