@@ -1,7 +1,7 @@
 /*
    GNUstep ProjectCenter - http://www.gnustep.org/experience/ProjectCenter.html
 
-   Copyright (C) 2000-2004 Free Software Foundation
+   Copyright (C) 2000-2009 Free Software Foundation
 
    Authors: Philippe C.D. Robert
             Serg Stoyan
@@ -159,16 +159,16 @@
   switch([sender indexOfSelectedItem]) 
     {
     case 0:
-      [inspectorView setContentView: buildAttributesView];
+      [inspectorView setContentView:buildAttributesView];
       break;
     case 1:
-      [inspectorView setContentView: projectAttributesView];
+      [inspectorView setContentView:projectAttributesView];
       break;
     case 2:
-      [inspectorView setContentView: projectDescriptionView];
+      [inspectorView setContentView:projectDescriptionView];
       break;
     case 3:
-      [inspectorView setContentView: fileAttributesView];
+      [inspectorView setContentView:fileAttributesView];
       break;
     }
 
@@ -377,6 +377,9 @@
   // Table
   [searchOrderList setCornerView:nil];
   [searchOrderList setHeaderView:nil];
+  [searchOrderList setTarget:self];
+  [searchOrderList setAction:@selector(searchOrderClick:)];
+//  [searchOrderColumn setEditable:NO];
 
   // Buttons
   [self setSearchOrderButtonsState];
@@ -390,23 +393,29 @@
 {
   NSString *selectedTitle = [sender titleOfSelectedItem];
   
-  if ([selectedTitle isEqualToString: @"Header Directories Search Order"])
+  if ([selectedTitle isEqualToString:@"Header Directories Search Order"])
     {
       ASSIGN(searchItems, searchHeaders);
     }
-  else if ([selectedTitle isEqualToString: @"Library Directories Search Order"])
+  else if ([selectedTitle isEqualToString:@"Library Directories Search Order"])
     {
       ASSIGN(searchItems, searchLibs);
     }
-  else
+  else if ([selectedTitle isEqualToString:@"Build Targets"])
+    {
+      ASSIGN(searchItems,[project buildTargets]);
+    }
+  else  
     {
       ASSIGN(searchItems,nil);
     }
 
+  [searchOrderList reloadData];
+  [searchOrderList deselectAll:self];
+  [searchOrderTF setStringValue:@""];
+
   // Enable/disable buttons according to selected/not selected item
   [self setSearchOrderButtonsState];
-
-  [searchOrderList reloadData];
 }
 
 - (void)searchOrderDoubleClick:(id)sender
@@ -415,18 +424,28 @@
 
 - (void)searchOrderClick:(id)sender
 {
-  // Warning! NSTableView doesn't call action method
-  // TODO: Fix NSTableView (NSCell/NSActionCell?)
+  int row = [searchOrderList selectedRow];
+  [searchOrderTF setStringValue:[searchItems objectAtIndex:row]];
+  [searchOrderTF selectText:self];
   [self setSearchOrderButtonsState];
 }
 
 - (void)setSearchOrderButtonsState
 {
-  // Disable until implemented
+  // "Set..." button is always off until functionality will be implemented
   [searchOrderSet setEnabled:NO];
 
-  return; // See searchOrderClick
-  
+  // After loadable inspectors implementation make it work by
+  // detection of text field becoming first responder.
+/*  if ([inspectorPanel firstResponder] == searchOrderTF)
+    {
+      [searchOrderAdd setEnabled:YES];
+    }
+  else
+    {
+      [searchOrderAdd setEnabled:NO];
+    }*/
+
   if ([searchOrderList selectedRow] == -1)
     {
       [searchOrderRemove setEnabled:NO];
@@ -458,6 +477,11 @@
 {
   NSString *value = [searchOrderTF stringValue];
 
+  if ([value isEqualToString:@""])
+    {
+      return;
+    }
+
   [searchItems addObject:value];
   [searchOrderTF setStringValue:@""];
   [self syncSearchOrder];
@@ -472,17 +496,20 @@
   pIndex = [searchOrderPopup indexOfSelectedItem];
   switch (pIndex)
     {
-    case 0:
+    case 0: // Headers
       [project setProjectDictObject:searchItems
                              forKey:PCSearchHeaders
 			     notify:YES];
       break;
-    case 1:
+    case 1: // Libraries
       [project setProjectDictObject:searchItems
                              forKey:PCSearchLibs
 			     notify:YES];
       break;
-    case 2:
+    case 2: // Targets
+      [project setProjectDictObject:searchItems
+                             forKey:PCBuilderTargets
+			     notify:YES];
       return;
     }
 }
@@ -555,6 +582,7 @@
   [authorsList setHeaderView:nil];
   [authorsList addTableColumn:authorsColumn];
   [authorsList setDataSource:self];
+  [authorsList setDelegate:self];
 
   //
   [authorsScroll setDocumentView:authorsList];
@@ -933,4 +961,12 @@
     }
 }
 
+- (void) tableView: (NSTableView*)aTableView
+   willDisplayCell: (id)aCell
+    forTableColumn: (NSTableColumn*)aTableColumn
+               row: (int)rowIndex
+{
+  [(NSTextFieldCell *)aCell setScrollable:YES];
+}
+
 @end
