@@ -23,27 +23,29 @@
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
 */
 
-#include <ProjectCenter/PCDefines.h>
-#include <ProjectCenter/PCPrefController.h>
-#include <ProjectCenter/PCLogController.h>
+#import <ProjectCenter/PCDefines.h>
+#import <ProjectCenter/PCLogController.h>
 
-#include <ProjectCenter/PCBundleManager.h>
-#include <ProjectCenter/PCFileManager.h>
-#include <ProjectCenter/PCFileCreator.h>
-#include <ProjectCenter/PCEditorManager.h>
-#include <ProjectCenter/PCProjectManager.h>
+#import <ProjectCenter/PCBundleManager.h>
+#import <ProjectCenter/PCFileManager.h>
+#import <ProjectCenter/PCFileCreator.h>
+#import <ProjectCenter/PCEditorManager.h>
+#import <ProjectCenter/PCProjectManager.h>
 
-#include <ProjectCenter/PCProject.h>
-#include <ProjectCenter/PCProjectWindow.h>
-#include <ProjectCenter/PCProjectBrowser.h>
-#include <ProjectCenter/PCProjectInspector.h>
-#include <ProjectCenter/PCProjectEditor.h>
-#include <ProjectCenter/PCProjectBuilderPanel.h>
-#include <ProjectCenter/PCProjectLauncherPanel.h>
-#include <ProjectCenter/PCProjectLoadedFilesPanel.h>
+#import <ProjectCenter/PCProject.h>
+#import <ProjectCenter/PCProjectWindow.h>
+#import <ProjectCenter/PCProjectBrowser.h>
+#import <ProjectCenter/PCProjectInspector.h>
+#import <ProjectCenter/PCProjectEditor.h>
+#import <ProjectCenter/PCProjectBuilderPanel.h>
+#import <ProjectCenter/PCProjectLauncherPanel.h>
+#import <ProjectCenter/PCProjectLoadedFilesPanel.h>
 
-#include "Protocols/ProjectType.h"
-#include "Protocols/CodeEditor.h"
+#import "Protocols/ProjectType.h"
+#import "Protocols/CodeEditor.h"
+
+#import "Modules/Preferences/Saving/PCSavingPrefs.h"
+#import "Modules/Preferences/Misc/PCMiscPrefs.h"
 
 NSString *PCActiveProjectDidChangeNotification = @"PCActiveProjectDidChange";
 
@@ -75,15 +77,29 @@ NSString *PCActiveProjectDidChangeNotification = @"PCActiveProjectDidChange";
 	     object:nil];
 
       fileManager = [[PCFileManager alloc] initWithProjectManager:self];
-
     }
 
   return self;
 }
 
+- (BOOL)close
+{
+  if ([self closeAllProjects] == NO)
+    {
+      return NO;
+    }
+
+  if ((editorManager != nil) && ([editorManager closeAllEditors] == NO))
+    {
+      return NO;
+    }
+
+  return YES;
+}
+
 - (void)dealloc
 {
-#ifdef DEVELOPMENT
+#ifdef DEBUG
   NSLog (@"PCProjectManager: dealloc");
 #endif
 
@@ -102,12 +118,12 @@ NSString *PCActiveProjectDidChangeNotification = @"PCActiveProjectDidChange";
   RELEASE(projectTypeAccessaryView);
   RELEASE(fileTypeAccessaryView);
 
-  if (editorManager)    RELEASE(editorManager);
+  TEST_RELEASE(editorManager);
 
-  if (projectInspector) RELEASE(projectInspector);
-  if (loadedFilesPanel) RELEASE(loadedFilesPanel);
-  if (buildPanel)       RELEASE(buildPanel);
-  if (launchPanel)      RELEASE(launchPanel);
+  TEST_RELEASE(projectInspector);
+  TEST_RELEASE(loadedFilesPanel);
+  TEST_RELEASE(buildPanel);
+  TEST_RELEASE(launchPanel);
 
   [super dealloc];
 }
@@ -127,7 +143,7 @@ NSString *PCActiveProjectDidChangeNotification = @"PCActiveProjectDidChange";
   prefController = aController;
 }
 
-- (id)prefController
+- (id <PCPreferences>)prefController
 {
   return prefController;
 }
@@ -198,8 +214,7 @@ NSString *PCActiveProjectDidChangeNotification = @"PCActiveProjectDidChange";
 {
   NSTimeInterval interval;
 
-  interval = [[[PCPrefController sharedPCPreferences] 
-    objectForKey:AutoSavePeriod] intValue];
+  interval = [[prefController stringForKey:AutoSavePeriod] intValue];
 
   if (interval > 0 && saveTimer == nil)
     {
@@ -282,10 +297,7 @@ NSString *PCActiveProjectDidChangeNotification = @"PCActiveProjectDidChange";
 
 - (NSPanel *)loadedFilesPanel
 {
-  NSUserDefaults *ud = [NSUserDefaults standardUserDefaults];
-
-  if (!loadedFilesPanel
-      && [[ud objectForKey:SeparateLoadedFiles] isEqualToString:@"YES"])
+  if (!loadedFilesPanel && [prefController boolForKey:UseTearOffWindows])
     {
       loadedFilesPanel = 
 	[[PCProjectLoadedFilesPanel alloc] initWithProjectManager:self];
@@ -296,19 +308,15 @@ NSString *PCActiveProjectDidChangeNotification = @"PCActiveProjectDidChange";
 
 - (void)showProjectLoadedFiles:(id)sender
 {
-  if ([[[[NSUserDefaults standardUserDefaults] dictionaryRepresentation]
-              objectForKey: SeparateLoadedFiles] isEqualToString: @"YES"])
+  if ([prefController boolForKey:UseTearOffWindows])
     {
-      [[self loadedFilesPanel] orderFront: nil];
+      [[self loadedFilesPanel] orderFront:nil];
     }
 }
 
 - (NSPanel *)buildPanel
 {
-  NSUserDefaults *ud = [NSUserDefaults standardUserDefaults];
-
-  if (!buildPanel
-      && [[ud objectForKey:SeparateBuilder] isEqualToString:@"YES"])
+  if (!buildPanel && [prefController boolForKey:UseTearOffWindows])
     {
       buildPanel = [[PCProjectBuilderPanel alloc] initWithProjectManager:self];
     }
@@ -318,10 +326,7 @@ NSString *PCActiveProjectDidChangeNotification = @"PCActiveProjectDidChange";
 
 - (NSPanel *)launchPanel
 {
-  NSUserDefaults *ud = [NSUserDefaults standardUserDefaults];
-
-  if (!launchPanel
-      && [[ud objectForKey:SeparateLauncher] isEqualToString:@"YES"])
+  if (!launchPanel && [prefController boolForKey:UseTearOffWindows])
     {
       launchPanel = [[PCProjectLauncherPanel alloc] initWithProjectManager:self];
     }
@@ -334,16 +339,6 @@ NSString *PCActiveProjectDidChangeNotification = @"PCActiveProjectDidChange";
   return findPanel;
 }
 
-- (NSString *)projectPath
-{
-  return [activeProject projectPath];
-}
-
-- (NSString *)selectedFileName
-{
-  return [[activeProject projectBrowser] nameOfSelectedFile];
-}
-
 // ============================================================================
 // ==== Project management
 // ============================================================================
@@ -390,12 +385,10 @@ NSString *PCActiveProjectDidChangeNotification = @"PCActiveProjectDidChange";
 
 - (void)saveAllProjectsIfNeeded
 {
-  NSUserDefaults *defs = [NSUserDefaults standardUserDefaults];
-
 //  PCLogInfo(self, @"saveAllProjectsIfNeeded");
 
   // If this method was called not by NSTimer, check if we should save projects
-  if ([[defs objectForKey:SaveOnQuit] isEqualToString:@"YES"])
+  if ([prefController boolForKey:SaveOnQuit])
     {
       [self saveAllProjects];
     }
@@ -424,6 +417,7 @@ NSString *PCActiveProjectDidChangeNotification = @"PCActiveProjectDidChange";
 // ==== Project actions
 // ============================================================================
 
+
 - (PCProject *)convertLegacyProject:(NSMutableDictionary *)pDict
                              atPath:(NSString *)aPath
 {
@@ -439,7 +433,7 @@ NSString *PCActiveProjectDidChangeNotification = @"PCActiveProjectDidChange";
   NSString               *_2file = nil;
   NSString               *_resFile = nil;
   unsigned               i = 0;
-  PCProject<ProjectType> *project = nil;
+  PCProject              *project = nil;
   NSMutableArray         *otherResArray = nil;
   NSString               *plistFile = nil;
 
@@ -453,7 +447,8 @@ NSString *PCActiveProjectDidChangeNotification = @"PCActiveProjectDidChange";
   NSLog(@"Convert legacy");
 
   // Gorm project type doesn't exists anymore
-  if ([projectClassName isEqualToString:@"PCGormProj"])
+  if ([projectClassName isEqualToString:@"PCGormProj"] ||
+      [projectClassName isEqualToString:@"PCAppProj"])
     {
       projectTypeName = [NSString stringWithString:@"Application"];
       projectClassName = [projectTypes objectForKey:projectTypeName];
@@ -545,88 +540,103 @@ NSString *PCActiveProjectDidChangeNotification = @"PCActiveProjectDidChange";
   return project;
 }
 
-- (PCProject *)loadProjectAt:(NSString *)aPath
+- (PCProject *)openProjectAt:(NSString *)aPath makeActive: (BOOL)flag
 {
-  NSMutableDictionary    *projectFile = nil;
-  NSString               *projectTypeName = nil;
-  NSString               *projectClassName = nil;
-  PCProject<ProjectType> *project = nil;
-
-  projectFile = [NSMutableDictionary dictionaryWithContentsOfFile:aPath];
-  
-  // For compatibility with 0.3.x projects
-  project = [self convertLegacyProject:projectFile atPath:aPath];
-  if (project)
-    {// Project was converted and created PC*Project with alloc&init
-      aPath = [[aPath stringByDeletingLastPathComponent]
-	stringByAppendingPathComponent:@"PC.project"];
-    }
-  else
-    {// No conversion were taken
-      projectTypeName = [projectFile objectForKey:PCProjectType];
-      projectClassName = [projectTypes objectForKey:projectTypeName];
-      if (projectClassName == nil)
-	{
-	  NSRunAlertPanel(@"Load Project",
-			  @"Project type '%@' is not supported!",
-			  @"OK",nil,nil,projectTypeName); 
-	  return nil;
-	}
-      project = [bundleManager objectForClassName:projectClassName
-				       bundleType:@"project"
-					 protocol:@protocol(ProjectType)];
-    }
-
-  if (![project openWithDictionaryAt:aPath]) 
-    {
-      NSRunAlertPanel(@"Load Project",
-    		      @"Could not load project '%@'!",
-    		      @"OK",nil,nil,aPath); 
-      return nil;
-    }
-
-  PCLogStatus(self, @"Project %@ loaded as %@", 
-	      [project projectName], [project projectTypeName]);
-
-  // Started only if there's not save timer yet
-  [self startSaveTimer];
-  [project validateProjectDict];
-
-  return project;
-}
-
-- (BOOL)openProjectAt:(NSString *)aPath
-{
-  NSDictionary *pDict = [NSDictionary dictionaryWithContentsOfFile:aPath];
-  NSString     *projectName = nil;
   PCProject    *project = nil;
   NSDictionary *wap = nil;
-  BOOL         isDir = NO;
-
-  projectName = [pDict objectForKey:PCProjectName];
-
-  if ((project = [loadedProjects objectForKey:projectName]) != nil)
-    {
-      [[project projectWindow] makeKeyAndOrderFront:self];
-      return YES;
-    }
-
-  if ([[NSFileManager defaultManager] fileExistsAtPath:aPath 
-                                           isDirectory:&isDir] && !isDir) 
-    {
-      project = [self loadProjectAt:aPath];
-
-      if (!project) 
+  
+  if ((project = [loadedProjects objectForKey: [aPath stringByDeletingLastPathComponent]]) == nil)
+    {  
+      NSMutableDictionary    *projectFile = nil;
+      NSString               *projectTypeName = nil;
+      NSString               *projectClassName = nil;
+      BOOL         isDir = NO;
+      BOOL         exists = [[NSFileManager defaultManager] fileExistsAtPath:aPath 
+							    isDirectory: &isDir];
+      if (!exists)
 	{
-	  return NO;
+	  return nil;
+	}
+      
+      if ([aPath rangeOfString: @" "].location != NSNotFound ||
+	  [aPath rangeOfString: @"\t"].location != NSNotFound ||
+	  [aPath rangeOfString: @"\r"].location != NSNotFound ||
+	  [aPath rangeOfString: @"\n"].location != NSNotFound)
+	{
+	  if (NSRunAlertPanel 
+	      (@"Open Project",
+	       @"Are you sure you want to open a project with whitespace in it's path?\n"
+	       @"GNUstep's build environment currently can't handle that reliably.",
+	       @"OK", @"Cancel", nil) != NSAlertDefaultReturn)
+	    {
+	      return nil;
+	    }
 	}
 
-      [loadedProjects setObject:project forKey:projectName];
-      [self setActiveProject:project];
+      if (!isDir)
+	{
+	  projectFile = [NSMutableDictionary dictionaryWithContentsOfFile: aPath];	  
+	}
+      else
+	{
+	  projectFile = [NSMutableDictionary dictionaryWithContentsOfFile: [aPath stringByAppendingPathComponent: @"PC.project"]];
+	}
+      
+      if(projectFile == nil)
+	{
+	  return nil;
+	}
+      
+      // For compatibility with 0.3.x projects
+      project = [self convertLegacyProject: projectFile atPath: aPath];
+      if (project)
+	{// Project was converted and created PC*Project with alloc&init
+	  aPath = [[aPath stringByDeletingLastPathComponent]
+		    stringByAppendingPathComponent:@"PC.project"];
+	}
+      else
+	{// No conversion were taken
+	  projectTypeName = [projectFile objectForKey:PCProjectType];
+	  projectClassName = [projectTypes objectForKey:projectTypeName];
+	  if (projectClassName == nil)
+	    {
+	      NSRunAlertPanel(@"Open Project",
+			      @"Project type '%@' is not supported!\n"
+			      @"Report the bug, please!",
+			      @"OK", nil, nil, projectTypeName); 
+	      return nil;
+	    }
+	}
+      
+      project = [bundleManager objectForClassName:projectClassName
+			       bundleType:@"project"
+			       protocol:@protocol(ProjectType)];
+      
+      if (!project || ![project openWithWrapperAt:aPath]) 
+	{
+	  NSRunAlertPanel(@"Open Project",
+			  @"Unable to open project '%@'.\nReport bug, please!",
+			  @"OK",nil,nil,aPath); 
+	  return nil;
+	}
+      
+      PCLogStatus(self, @"Project %@ loaded as %@", 
+		  [project projectName], [project projectTypeName]);
+      
+      // Started only if there's not save timer yet
+      [self startSaveTimer];
+      [project validateProjectDict];
+      
+      if (!project) 
+	{
+	  return nil;
+	}
+      
+      [loadedProjects setObject:project forKey: [project projectPath]];
       [project setProjectManager:self];
-
+      
       // Windows and panels
-      wap = [pDict objectForKey:@"PC_WINDOWS"];
+      wap = [projectFile objectForKey:PCWindows];
       if ([[wap allKeys] containsObject:@"ProjectBuild"])
 	{
 	  [[project projectWindow] showProjectBuild:self];
@@ -639,10 +649,44 @@ NSString *PCActiveProjectDidChangeNotification = @"PCActiveProjectDidChange";
 	{
 	  [[project projectWindow] showProjectLoadedFiles:self];
 	}
-      return YES;
+      
+      if (flag)
+	{
+	  [[project projectWindow] makeKeyAndOrderFront:self];
+	  [self setActiveProject: project];
+	}
+    }
+  
+  return project;
+}
+
+- (void)openProject
+{
+  NSArray  *files = nil;
+  NSString *filePath = nil, *temp = nil;
+  NSArray  *fileTypes = [NSArray arrayWithObjects:@"pcproj",@"project",nil];
+  NSEnumerator *en = nil;
+
+  files = [fileManager filesOfTypes:fileTypes
+			  operation:PCOpenProjectOperation
+			   multiple:NO
+			      title:@"Open Project"
+			    accView:nil];
+
+  filePath = [files objectAtIndex:0];
+  en = [files objectEnumerator];
+  while((temp = [en nextObject]) != nil)
+    {
+      if([[temp pathExtension] isEqual: @"pcproj"])
+	{
+	  filePath = temp;
+	}
     }
 
-  return NO;
+  if (filePath != nil)
+    {
+      [self openProjectAt:filePath makeActive: YES];
+    }
 }
 
 - (PCProject *)createProjectOfType:(NSString *)projectType 
@@ -651,9 +695,8 @@ NSString *PCActiveProjectDidChangeNotification = @"PCActiveProjectDidChange";
   NSString               *className = [projectTypes objectForKey:projectType];
   PCProject<ProjectType> *projectCreator;
   PCProject              *project = nil;
-  NSString               *projectName = [aPath lastPathComponent];
 
-  if ((project = [loadedProjects objectForKey:projectName]) != nil)
+  if ((project = [loadedProjects objectForKey: [aPath stringByDeletingLastPathComponent]]) != nil)
     {
       [[project projectWindow] makeKeyAndOrderFront:self];
       return project;
@@ -665,18 +708,30 @@ NSString *PCActiveProjectDidChangeNotification = @"PCActiveProjectDidChange";
 //  NSLog(@"%@ CLASS: %@", className, projectCreator);
   if (!projectCreator)
     {
+      NSRunAlertPanel(@"New Project",
+		      @"Could not create project directory %@.\n"
+		      @"No project creator. Report the bug, please!",
+		      @"OK", nil, nil, aPath); 
       return nil;
     }
 
   // Create project directory
   if (![[PCFileManager defaultManager] createDirectoriesIfNeededAtPath:aPath])
     {
+      NSRunAlertPanel(@"New Project",
+		      @"Could not create project directory %@.\n"
+		      @"Check permissions of the directory where you"
+		      @" want to create a project",
+		      @"OK", nil, nil, aPath); 
       return nil;
     }
 
   // Create project
   if (!(project = [projectCreator createProjectAt:aPath])) 
     {
+      NSRunAlertPanel(@"New Project",
+		      @"Project %@ could not be created.\nReport bug, please!",
+		      @"OK",nil,nil,[project projectName]); 
       return nil;
     }
 
@@ -686,31 +741,6 @@ NSString *PCActiveProjectDidChangeNotification = @"PCActiveProjectDidChange";
   return project;
 }
 
-- (void)openProject
-{
-  NSArray  *files = nil;
-  NSString *filePath = nil;
-  NSArray  *fileTypes = [NSArray arrayWithObjects:@"project",@"pcproj",nil];
-
-  files = [fileManager filesOfTypes:fileTypes
-			  operation:PCOpenProjectOperation
-			   multiple:NO
-			      title:@"Open Project"
-			    accView:nil];
-  filePath = [files objectAtIndex:0];
-
-  if (filePath != nil)
-    {
-      [self openProjectAt:filePath];
-      [[activeProject projectWindow] makeKeyAndOrderFront:self];
-
-/*      NSRunAlertPanel(@"Attention!",
-		      @"Couldn't open project %@!",
-		      @"OK",nil,nil,
-		      [filePath stringByDeletingLastPathComponent]);*/
-    }
-}
-
 - (void)newProject
 {
   NSArray   *files;
@@ -729,16 +759,31 @@ NSString *PCActiveProjectDidChangeNotification = @"PCActiveProjectDidChange";
   filePath = [files objectAtIndex:0];
   if (filePath != nil) 
     {
+      if ([filePath rangeOfString: @" "].location != NSNotFound ||
+	  [filePath rangeOfString: @"\t"].location != NSNotFound ||
+	  [filePath rangeOfString: @"\r"].location != NSNotFound ||
+	  [filePath rangeOfString: @"\n"].location != NSNotFound)
+	{
+	  if (NSRunAlertPanel 
+	      (@"New Project",
+	       @"Are you sure you want to create a project with whitespace in it's path?\n"
+	       @"GNUstep's build environment currently can't handle that reliably.",
+	       @"OK", @"Cancel", nil) != NSAlertDefaultReturn)
+	    {
+	      return;
+	    }
+	}
+      
       projectType = [projectTypePopup titleOfSelectedItem];
 
       if (!(project = [self createProjectOfType:projectType path:filePath]))
 	{
-	  NSRunAlertPanel(@"Attention!",
-			  @"Failed to create %@!",
-			  @"OK",nil,nil,filePath);
+	  // No need to open alert panel. Alert panel was already opened
+	  // in createProjectOfType:path: method.
+	  return;
 	}
 
-      [loadedProjects setObject:project forKey:[project projectName]];
+      [loadedProjects setObject:project forKey: [project projectPath]];
       [self setActiveProject:project];
       [[project projectWindow] orderFront:self];
     }
@@ -758,9 +803,9 @@ NSString *PCActiveProjectDidChangeNotification = @"PCActiveProjectDidChange";
   // Save PC.project and the makefiles!
   if ([rootProject save] == NO)
     {
-      NSRunAlertPanel(@"Attention!",
+      NSRunAlertPanel(@"Save Project",
 		      @"Couldn't save project %@!", 
-		      @"OK",nil,nil,[activeProject projectName]);
+		      @"OK", nil, nil, [rootProject projectName]);
       return NO;
     }
 
@@ -829,6 +874,9 @@ NSString *PCActiveProjectDidChangeNotification = @"PCActiveProjectDidChange";
   NSMutableArray *subprojs = [NSMutableArray array];
   unsigned       i;
 
+  NSLog(@"Root active project '%@' category '%@'", 
+	[project projectName], category);
+
   // Determining target project
   if ([categoryKey isEqualToString:PCSubprojects])
     {
@@ -856,7 +904,7 @@ NSString *PCActiveProjectDidChangeNotification = @"PCActiveProjectDidChange";
 
       if ([categoryKey isEqualToString:PCLibraries])
 	{
-	  ret = NSRunAlertPanel(@"Remove",
+	  ret = NSRunAlertPanel(@"Remove File",
 				@"Remove libraries from Project?",
 				@"Remove",
 				@"Cancel",
@@ -864,7 +912,7 @@ NSString *PCActiveProjectDidChangeNotification = @"PCActiveProjectDidChange";
 	}
       else
 	{
-	  ret = NSRunAlertPanel(@"Remove",
+	  ret = NSRunAlertPanel(@"Remove File",
 				removeString,
 				@"...from Project and Disk",
 				@"...from Project only",
@@ -899,7 +947,7 @@ NSString *PCActiveProjectDidChangeNotification = @"PCActiveProjectDidChange";
 
 	  if (!ret)
 	    {
-	      NSRunAlertPanel(@"Alert",
+	      NSRunAlertPanel(@"Remove File",
 			      @"Error removing files from project %@!",
 			      @"OK", nil, nil, [activeProject projectName]);
 	      return NO;
@@ -907,7 +955,7 @@ NSString *PCActiveProjectDidChangeNotification = @"PCActiveProjectDidChange";
 	  else if (flag) 
 	    {
 	      // Save project because we've removed file(s) from disk
-	      // Should be fixed later (add pending removal of files?)
+	      // TODO: Maybe fix it later? (add pending removal of files)
 	      [activeProject save];
 	    }
 	}
@@ -923,16 +971,15 @@ NSString *PCActiveProjectDidChangeNotification = @"PCActiveProjectDidChange";
 - (void)closeProject:(PCProject *)aProject
 {
   PCProject *currentProject = nil;
-  NSString  *projectName = [aProject projectName];
 
-  currentProject = [loadedProjects objectForKey:projectName];
+  currentProject = [loadedProjects objectForKey: [aProject projectPath]];
   if (!currentProject)
     {
       return;
     }
 
   // Remove it from the loaded projects!
-  [loadedProjects removeObjectForKey:projectName];
+  [loadedProjects removeObjectForKey: [aProject projectPath]];
 
   if ([loadedProjects count] == 0)
     {
@@ -965,9 +1012,7 @@ NSString *PCActiveProjectDidChangeNotification = @"PCActiveProjectDidChange";
 
 - (void)closeProject
 {
-  NSUserDefaults *defs = [NSUserDefaults standardUserDefaults];
-
-  if ([[defs objectForKey:SaveOnQuit] isEqualToString:@"YES"])
+  if ([prefController boolForKey:SaveOnQuit])
     {
       [activeProject save];
     }
@@ -979,14 +1024,13 @@ NSString *PCActiveProjectDidChangeNotification = @"PCActiveProjectDidChange";
 {
   PCProject      *project = nil;
   NSEnumerator   *enumerator = [loadedProjects objectEnumerator];
-  NSUserDefaults *defs = [NSUserDefaults standardUserDefaults];
 
 //  PCLogInfo(self, @"loaded %i projects", [loadedProjects count]);
 
   while ([loadedProjects count] > 0)
     {
       project = [enumerator nextObject];
-      if ([[defs objectForKey:SaveOnQuit] isEqualToString:@"YES"])
+      if ([prefController boolForKey:SaveOnQuit])
 	{
 	  [project save];
 	}
@@ -1005,11 +1049,14 @@ NSString *PCActiveProjectDidChangeNotification = @"PCActiveProjectDidChange";
 
 - (void)openFileAtPath:(NSString *)filePath
 {
+  editorManager = [self editorManager];
+
   if (filePath != nil)
     {
-      [[self editorManager] openEditorForFile:filePath 
-				     editable:YES
-				     windowed:YES];
+      [editorManager openEditorForFile:filePath 
+			      editable:YES
+			      windowed:YES];
+      [editorManager orderFrontEditorForFile:filePath];
     }
 }
 
@@ -1024,7 +1071,10 @@ NSString *PCActiveProjectDidChangeNotification = @"PCActiveProjectDidChange";
 			      title:@"Open File"
 			    accView:nil];
   filePath = [files objectAtIndex:0];
-  [self openFileAtPath:filePath];
+  if (filePath)
+    {
+      [self openFileAtPath:filePath];
+    }
 }
 
 - (void)newFile
@@ -1034,7 +1084,16 @@ NSString *PCActiveProjectDidChangeNotification = @"PCActiveProjectDidChange";
 
 - (BOOL)saveFile
 {
-  return [[activeProject projectEditor] saveFile];
+  if (activeProject) 
+    {
+      return [[[activeProject projectEditor] activeEditor] saveFile];
+    }
+  else if (editorManager)
+    {
+      return [[editorManager activeEditor] saveFile];
+    }
+
+  return NO;
 }
 
 - (BOOL)saveFileAs
@@ -1045,13 +1104,14 @@ NSString *PCActiveProjectDidChangeNotification = @"PCActiveProjectDidChange";
   files = [fileManager filesOfTypes:nil
 			  operation:PCSaveFileOperation
 			   multiple:NO
-			      title:@"Save As..."
+			      title:@"Save File As..."
 			    accView:nil];
   filePath = [files objectAtIndex:0];
 
   if (filePath != nil && ![[activeProject projectEditor] saveFileAs:filePath]) 
     {
-      NSRunAlertPanel(@"Alert", @"Couldn't save file to\n%@!",
+      NSRunAlertPanel(@"Save File As", 
+		      @"Unable to save file as\n%@!",
 		      @"OK", nil, nil, filePath);
       return NO;
     }
@@ -1087,13 +1147,14 @@ NSString *PCActiveProjectDidChangeNotification = @"PCActiveProjectDidChange";
   files = [fileManager filesOfTypes:nil
 			  operation:PCSaveFileOperation
 			   multiple:NO
-			      title:@"Save To..."
+			      title:@"Save File To..."
 			    accView:nil];
   filePath = [files objectAtIndex:0];
 
   if (filePath != nil && ![[activeProject projectEditor] saveFileTo:filePath]) 
     {
-      NSRunAlertPanel(@"Alert", @"Couldn't save file to\n%@!",
+      NSRunAlertPanel(@"Save File To", 
+		      @"Unable to save file to\n%@!",
 		      @"OK", nil, nil, filePath);
       return NO;
     }
@@ -1124,7 +1185,7 @@ NSString *PCActiveProjectDidChangeNotification = @"PCActiveProjectDidChange";
 
 @implementation PCProjectManager (Subprojects)
 
-- (BOOL)newSubproject
+- (BOOL)openNewSubprojectPanel
 {
 //  PCLogInfo(self, @"newSubproject");
 
@@ -1132,7 +1193,10 @@ NSString *PCActiveProjectDidChangeNotification = @"PCActiveProjectDidChange";
     {
       if ([NSBundle loadNibNamed:@"NewSubproject" owner:self] == NO)
 	{
-	  PCLogError(self, @"error loading NewSubproject NIB!");
+	  NSRunAlertPanel(@"New Subproject",
+			  @"Internal error!"
+			  @" Install ProjectCenter again, please.",
+			  @"OK", nil, nil);
 	  return NO;
 	}
 
@@ -1157,28 +1221,36 @@ NSString *PCActiveProjectDidChangeNotification = @"PCActiveProjectDidChange";
   [nsNameField setStringValue:@""];
   [nsPanel makeFirstResponder:nsNameField];
 
+  [nsPanel setLevel:NSModalPanelWindowLevel];
+  [NSApp runModalForWindow:nsPanel];
+
   return YES;
 }
 
 - (void)closeNewSubprojectPanel:(id)sender
 {
   [nsPanel orderOut:self];
+  [NSApp stopModal];
 }
 
-- (BOOL)createSubproject:(id)sender
-{
-  [nsPanel orderOut:self];
-
-  return [self createSubproject];
-}
-
-- (BOOL)createSubproject
+- (void)createSubproject:(id)sender
 {
   PCProject *subproject = nil;
   NSString  *spName = [nsNameField stringValue];
   NSString  *spPath = nil;
   NSString  *spType = [nsTypePB titleOfSelectedItem];
 
+  // Check if subproject with entered name already exists.
+  if (![activeProject doesAcceptFile:spName forKey:PCSubprojects])
+    {
+      NSRunAlertPanel(@"New Subproject",
+		      @"Subproject with name %@ already exists in project %@",
+		      @"OK", nil, nil, spName, [activeProject projectName]);
+      return;
+    }
+
+  [self closeNewSubprojectPanel:self];
+
   if (![[spName pathExtension] isEqualToString:@"subproj"])
     {
       spName = [[nsNameField stringValue] 
@@ -1193,7 +1265,7 @@ NSString *PCActiveProjectDidChangeNotification = @"PCActiveProjectDidChange";
   // Create subproject
   subproject = [self createSubprojectOfType:spType path:spPath];
 
-  return YES;
+  return;
 }
 
 - (PCProject *)createSubprojectOfType:(NSString *)projectType 
@@ -1202,19 +1274,16 @@ NSString *PCActiveProjectDidChangeNotification = @"PCActiveProjectDidChange";
   NSString               *className = [projectTypes objectForKey:projectType];
   PCProject<ProjectType> *projectCreator;
   PCProject              *subproject = nil;
-/*  NSString  *subprojectName = [aPath lastPathComponent];
-
-  if ((project = [activeProject objectForKey:projectName]) != nil)
-    {
-      [[project projectWindow] makeKeyAndOrderFront:self];
-      return project;
-    }*/
 
   projectCreator = [bundleManager objectForClassName:className 
 					  bundleType:@"project"
 					    protocol:@protocol(ProjectType)];
   if (!(subproject = [projectCreator createProjectAt:aPath])) 
     {
+      NSRunAlertPanel(@"New Subproject",
+		      @"Internal error!"
+		      @" Install ProjectCenter again, please.",
+		      @"OK", nil, nil);
       return nil;
     }
   [subproject setIsSubproject:YES];
@@ -1229,13 +1298,18 @@ NSString *PCActiveProjectDidChangeNotification = @"PCActiveProjectDidChange";
 
 - (void)controlTextDidChange:(NSNotification *)aNotif
 {
+  NSString *tfString = nil;
+  NSArray  *subprojectList = nil;
+
   if ([aNotif object] != nsNameField)
     {
       return;
     }
     
-  // TODO: Add check for valid subproject named
-  if ([[nsNameField stringValue] length] > 0)
+  // Check for valid subproject names
+  tfString = [nsNameField stringValue];
+  subprojectList = [[activeProject projectDict] objectForKey:PCSubprojects];
+  if (![subprojectList containsObject:tfString])
     {
       [nsCreateButton setEnabled:YES];
     }
