@@ -24,7 +24,6 @@
 */
 
 #import <ProjectCenter/PCDefines.h>
-#import <ProjectCenter/PCSplitView.h>
 #import <ProjectCenter/PCButton.h>
 
 #import <ProjectCenter/PCProjectManager.h>
@@ -38,9 +37,13 @@
 #import <ProjectCenter/PCProjectLoadedFiles.h>
 #import <ProjectCenter/PCProjectInspector.h>
 
-#import <ProjectCenter/PCPrefController.h>
+#import <Protocols/Preferences.h>
 #import <ProjectCenter/PCLogController.h>
 
+#import "Modules/Preferences/Misc/PCMiscPrefs.h"
+
+#import <math.h>
+
 @implementation PCProjectWindow
 
 // ============================================================================
@@ -65,6 +68,7 @@
 
 - (void)awakeFromNib
 {
+  id <PCPreferences> prefs = [[project projectManager] prefController];
   NSRect rect;
 
   [buildButton setToolTip:@"Project Builder"];
@@ -79,7 +83,7 @@
 
   [loadedFilesButton setToolTip:@"Loaded Files"];
   [loadedFilesButton setImage:IMAGE(@"Files")];
-  if ([self hasLoadedFilesView])
+  if (![prefs boolForKey:UseTearOffWindows])
     {
       [loadedFilesButton setEnabled:NO];
     }
@@ -114,7 +118,7 @@
     {
       rect.size.height = 185;
     }
-  v_split = [[PCSplitView alloc] initWithFrame:rect];
+  v_split = [[NSSplitView alloc] initWithFrame:rect];
   [v_split setAutoresizingMask:(NSViewWidthSizable | NSViewHeightSizable)];
   [v_split setVertical:YES];
   [v_split setDelegate:self];
@@ -128,7 +132,7 @@
   /*
    * LoadedFiles
    */
-  if ([self hasLoadedFilesView])
+  if (![prefs boolForKey:UseTearOffWindows])
     {
       [self showProjectLoadedFiles:self];
     }
@@ -140,10 +144,7 @@
    * Custom view
    * View where non-separated Builder, Launcher, Editor goes.
    */ 
-  if ([self hasCustomView])
-    {
-      [self _createCustomView];
-    }
+  [self _createCustomView];
 }
 
 - (id)initWithProject:(PCProject *)owner 
@@ -170,7 +171,7 @@
       // Window
       [projectWindow setFrameAutosaveName:@"ProjectWindow"];
 
-      pcWindows = [[project projectDict] objectForKey:@"PC_WINDOWS"];
+      pcWindows = [[project projectDict] objectForKey:PCWindows];
       windowFrame = [pcWindows objectForKey:@"ProjectWindow"];
 //      PCLogInfo(self, @"window frame %@", windowFrame);
       if (windowFrame != nil)
@@ -269,45 +270,11 @@
 // ==== Accessory methods
 // ============================================================================
 
-- (BOOL)hasCustomView
-{
-  NSUserDefaults *ud = [NSUserDefaults standardUserDefaults];
-
-  _hasCustomView = NO;
-  
-  if (![[ud objectForKey:SeparateEditor] isEqualToString:@"YES"]
-      && [[ud objectForKey:Editor] isEqualToString:@"ProjectCenter"])
-    {
-      _hasCustomView = YES;
-    }
-  if (![[ud objectForKey:SeparateBuilder] isEqualToString:@"YES"])
-    {
-      _hasCustomView = YES;
-    }
-  if (![[ud objectForKey:SeparateLauncher] isEqualToString:@"YES"])
-    {
-      _hasCustomView = YES;
-    }
-
-  return _hasCustomView;
-}
-
-- (BOOL)hasLoadedFilesView
-{
-  NSUserDefaults *ud = [NSUserDefaults standardUserDefaults];
-
-  if (![[ud objectForKey:SeparateLoadedFiles] isEqualToString:@"YES"])
-    {
-      _hasLoadedFilesView = YES;
-    }
-  else
-    {
-      _hasLoadedFilesView = NO;
-    }
-
-  return _hasLoadedFilesView;
-}
-
+// TODO: Should be removed when two modes will be implemented:
+// Build, Launch and Loaded Files are tear-off and tear-on.
+// Custom view is always shown because editor always opened on
+// one click in Browser. External editor will be opened only on
+// double click.
 - (NSView *)customContentView
 {
   return [customView contentView];
@@ -335,21 +302,30 @@
 
 - (void)showProjectBuild:(id)sender
 {
-  NSView  *view = [[project projectBuilder] componentView];
-  NSPanel *buildPanel = [[project projectManager] buildPanel];
+  id <PCPreferences> prefs = [[project projectManager] prefController];
+  NSView             *view = [[project projectBuilder] componentView];
+  NSPanel            *buildPanel = [[project projectManager] buildPanel];
+  PCProject          *rootActiveProject;
+
+  rootActiveProject = [[project projectManager] rootActiveProject];
   
-  if ([[[PCPrefController sharedPCPreferences] objectForKey:SeparateBuilder]
-      isEqualToString: @"YES"])
+//  NSLog(@"ProjectWindow showProjectBuild: componentView RC:%i", 
+//	[view retainCount]);
+
+  if ([prefs boolForKey:UseTearOffWindows])
     {
       if ([customView contentView] == view)
 	{
 	  [self showProjectEditor:self];
 	}
-      [buildPanel orderFront:nil];
+      if (rootActiveProject == project)
+	{
+	  [buildPanel orderFront:nil];
+	}
     }
   else
     {
-      if ([buildPanel isVisible])
+      if ((rootActiveProject == project) && [buildPanel isVisible])
 	{
 	  [buildPanel close];
 	}
@@ -359,20 +335,20 @@
 
 - (void)showProjectLaunch:(id)sender
 {
-  NSView  *view = nil;
-  NSPanel *launchPanel = nil;
+  id <PCPreferences> prefs = [[project projectManager] prefController];
+  NSView             *view = nil;
+  NSPanel            *launchPanel = nil;
 
   view = [[project projectLauncher] componentView];
   launchPanel = [[project projectManager] launchPanel];
 
-  if ([[[[NSUserDefaults standardUserDefaults] dictionaryRepresentation]
-           objectForKey: SeparateLauncher] isEqualToString: @"YES"])
+  if ([prefs boolForKey:UseTearOffWindows])
     {
       if ([customView contentView] == view)
 	{
 	  [self showProjectEditor:self];
 	}
-      [launchPanel orderFront: nil];
+      [launchPanel orderFront:nil];
     }
   else
     {
@@ -386,29 +362,37 @@
 
 - (void)showProjectLoadedFiles:(id)sender
 {
-  NSPanel      *panel = [[project projectManager] loadedFilesPanel];
-  NSScrollView *componentView;
+  id <PCPreferences> prefs = [[project projectManager] prefController];
+  NSPanel            *panel = [[project projectManager] loadedFilesPanel];
+  PCProject          *rootActiveProject;
+  NSScrollView       *componentView;
 
+  rootActiveProject = [[project projectManager] rootActiveProject];
   componentView = (NSScrollView *)[[project projectLoadedFiles] componentView];
       
 //  PCLogInfo(self, @"showProjectLoadedFiles");
 
-  if ([self hasLoadedFilesView])
+  if ([prefs boolForKey:UseTearOffWindows])
     {
-      if (panel && [panel isVisible])
+      [componentView setBorderType:NSNoBorder];
+      [componentView removeFromSuperview];
+      if (rootActiveProject == project)
+	{
+	  [panel orderFront:nil];
+	}
+      [v_split adjustSubviews];
+    }
+  else
+    {
+      if ((rootActiveProject == project) && panel && [panel isVisible])
 	{
 	  [panel close];
 	}
 
       [componentView setBorderType:NSBezelBorder];
-      [componentView setFrame:NSMakeRect(0,0,128,130)];
-      [v_split addSubview:[[project projectLoadedFiles] componentView]];
-      [v_split adjustSubviews];
-    }
-  else
-    {
-      [componentView setBorderType:NSNoBorder];
-      [panel orderFront:nil];
+      // Preserve width of view in panel
+//      [componentView setFrame:NSMakeRect(0,0,128,130)];
+      [v_split addSubview:componentView];
       [v_split adjustSubviews];
     }
 }
@@ -555,80 +539,62 @@
   [self makeKeyWindow];*/
 }
 
+// TODO: Review determining of tear-off panels. Current implementation
+// loads Build, Launch and Loaded Files to check visibility status.
+// It is incorrect behaviour; Build. Launch and LoadedFiles initialized
+// even if not in use.
 - (void)preferencesDidChange:(NSNotification *)aNotif
 {
-  NSDictionary *prefsDict = [[aNotif object] preferencesDict];
+  id <PCPreferences> prefs = [aNotif object];
  
-  PCLogStatus(self, @"Preferences did change");
- 
-  //--- Add Custom view
-  if ([self hasCustomView] && customView == nil)
-    {
-      [self _createCustomView];
-    }
-  //--- Remove Custom view
-  if (![self hasCustomView] && customView != nil)
-    {
-      [customView removeFromSuperview];
-      [h_split adjustSubviews];
-      customView = nil;
-    }
+  NSLog(@"ProjectWindow: Preferences did change");
 
-
-  // Project Builder
-  if ([[prefsDict objectForKey:@"SeparateBuilder"] isEqualToString:@"YES"])
+/*  if ([[project projectManager] rootActiveProject] != project)
     {
-      // Project Build is sepearate and visible in project window
+      return;
+    }*/
+
+  if ([prefs boolForKey:UseTearOffWindows])
+    {
+      // Project Build 
       if ([[[project projectBuilder] componentView] window] == projectWindow)
 	{
 	  [self showProjectBuild:self];
 	}
-    }
-  else
-    {
-      NSPanel *buildPanel = [[project projectManager] buildPanel];
-      
-      if ([buildPanel isVisible] == YES)
-	{
-	  [self showProjectBuild:self];
-	}
-    }
-
-  // Project Launcher
-  if ([[prefsDict objectForKey:@"SeparateLauncher"] isEqualToString:@"YES"])
-    {
+      // Project Launch
       if ([[[project projectLauncher] componentView] window] == projectWindow)
 	{
 	  [self showProjectLaunch:self];
 	}
-    }
-  else
-    {
-      NSPanel *launchPanel = [[project projectManager] launchPanel];
-      
-      if ([launchPanel isVisible] == YES)
-	{
-	  [self showProjectLaunch:self];
-	}
-    }
-
-  // Loaded Files view
-  if ([self hasLoadedFilesView])
-    {
-      if ([[v_split subviews] count] == 1)
-	{
-	  [self showProjectLoadedFiles:self];
-	}
-      [loadedFilesButton setEnabled:NO];
-    }
-  else 
-    {
+      // Loaded Files
       if ([[v_split subviews] count] == 2)
 	{
 	  [self showProjectLoadedFiles:self];
 	}
       [loadedFilesButton setEnabled:YES];
     }
+  else
+    {
+      NSPanel *buildPanel = [[project projectManager] buildPanel];
+      NSPanel *launchPanel = [[project projectManager] launchPanel];
+      
+      // Project Build 
+      if ([buildPanel isVisible] == YES)
+	{
+	  [self showProjectBuild:self];
+	}
+      // Project Launch
+      if ([launchPanel isVisible] == YES)
+	{
+	  [self showProjectLaunch:self];
+	}
+      // Loaded Files
+      if ([[v_split subviews] count] == 1)
+	{
+	  [self showProjectLoadedFiles:self];
+	}
+      [loadedFilesButton setEnabled:NO];
+    }
 }
 
 - (void)browserDidSetPath:(NSNotification *)aNotif
@@ -801,9 +767,16 @@
   NSDictionary *projectDict = nil;
   NSDictionary *windowsDict = nil;
   NSString     *browserString = nil;
-  NSRect       browserRect;
+  NSRect       browserRect = NSMakeRect(0,0,0,0);
   NSRect       boxRect;
 
+//  NSLog(@"resizeVerticalSubiewsWithOldSize entered split view width: %f, height %f", splitSize.width, splitSize.height);
+
+  boxRect = [v_split frame];
+  boxRect.size.width = floorf(boxRect.size.width);
+  boxRect.size.height = floorf(boxRect.size.height);
+  [v_split setFrame:boxRect];
+
   if (splitSize.width == oldSize.width && splitSize.height == oldSize.height)
     {
       return;
@@ -815,47 +788,63 @@
   if (!_splitViewsRestored)
     {
       projectDict = [project projectDict];
-      windowsDict = [projectDict objectForKey:@"PC_WINDOWS"];
+      windowsDict = [projectDict objectForKey:PCWindows];
       if (windowsDict != nil)
 	{
 	  browserString = [windowsDict objectForKey:@"ProjectBrowser"];
+	  NSLog(@"Browser size '%@'", browserString);
 	  if (browserString != nil && ![browserString isEqualToString:@""])
 	    {
 	      browserRect = NSRectFromString(browserString);
 	    }
 	}
-      else
+      // Unable to restore browser frame
+      if (browserRect.size.width == 0 && browserRect.size.height == 0)
 	{
-	  browserRect = NSMakeRect(0, 0, splitSize.width, splitSize.height);
+	  browserRect = NSMakeRect(0, 0, 
+				   floorf(splitSize.width), 
+				   floorf(splitSize.height));
 	}
+      [browserView setFrame:browserRect];
     }
 
   // Use saved frame of ProjectBrowser only first time. Every time window is
   // resized use new size of subviews.
-  if (_splitViewsRestored)
+/*  if (_splitViewsRestored)
     {
       browserRect = [[[project projectBrowser] view] frame];
       browserRect.size.height = splitSize.height;
-      if (![self hasLoadedFilesView])
+      if ([[prefs objectForKey:UseTearOffWindows] isEqualToString:@"YES"])
 	{
 	  browserRect.size.width = splitSize.width;
 	}
     }
 
   // Browser
-//  NSLog(@"browser %@", NSStringFromRect(browserRect));
-  [browserView setFrame:browserRect];
+  NSLog(@"%@ browser %@",
+	[project projectName], NSStringFromRect(browserRect));
+  [browserView setFrame:browserRect];*/
 
   // Loaded Files 
-  if ([self hasLoadedFilesView])
+/*  if ([[prefs objectForKey:UseTearOffWindows] isEqualToString:@"NO"])
     {
       boxRect.origin.x = browserRect.size.width + [v_split dividerThickness];
       boxRect.origin.y = 0;
       boxRect.size.width = [v_split frame].size.width - boxRect.origin.x;
       boxRect.size.height = [v_split frame].size.height;
-//      NSLog(@"loadedFiles %@", NSStringFromRect(boxRect));
+
+      if (boxRect.size.width < 60)
+	{
+	  boxRect.size.width = 60;
+	  boxRect.origin.x = [v_split frame].size.width - boxRect.size.width;
+	}
+
+      NSLog(@"%@ loadedFiles %@",
+	    [project projectName], NSStringFromRect(boxRect));
       [[[project projectLoadedFiles] componentView] setFrame:boxRect];
-    }
+    }*/
+
+  [v_split adjustSubviews];
 
   _splitViewsRestored = YES;
 }
@@ -868,6 +857,8 @@
   NSRect vSplitRect;
   NSRect boxRect;
   
+//  NSLog(@"resizeHorizontalSubiewsWithOldSize entered split view width: %f, height %f", splitSize.width, splitSize.height);
+
   if (splitSize.width == oldSize.width && splitSize.height == oldSize.height)
     {
       return;
@@ -881,25 +872,25 @@
   vSplitRect = [browserView frame];
   vSplitRect.origin.x = 0;
   vSplitRect.origin.y = 0;
-  if (![self hasCustomView])
+/*  if (![self hasCustomView])
     {
       vSplitRect.size = hSplitSize;
     }
   else
-    {
+    {*/
       vSplitRect.size.width = hSplitSize.width;
-    }
+/*    }*/
   [v_split setFrame:vSplitRect];
   
   // Custom view (Editor|Builder|Launcher)
-  if ([self hasCustomView])
-    {
+/*  if ([self hasCustomView])
+    {*/
       boxRect.origin.x = 0;
       boxRect.origin.y = vSplitRect.size.height + [h_split dividerThickness];
       boxRect.size.width = hSplitSize.width;
       boxRect.size.height = hSplitSize.height - boxRect.origin.y;
       [customView setFrame:boxRect];
-    }
+//    }
 }
 
 - (void)         splitView:(NSSplitView *)sender
