@@ -1,10 +1,11 @@
 /*
    GNUstep ProjectCenter - http://www.gnustep.org/experience/ProjectCenter.html
 
-   Copyright (C) 2002-2004 Free Software Foundation
+   Copyright (C) 2002-2010 Free Software Foundation
 
    Authors: Philippe C.D. Robert
             Serg Stoyan
+	    Riccardo Mottola
 
    This file is part of GNUstep.
 
@@ -25,8 +26,6 @@
 
 #import "PCEditor.h"
 #import "PCEditorView.h"
-//#import "CommandQueryPanel.h"
-//#import "LineQueryPanel.h"
 
 @implementation PCEditor (UInterface)
 
@@ -46,16 +45,17 @@
 
   windowWidth = [[NSFont userFixedPitchFontOfSize:0.0] widthOfString:@"A"];
   windowWidth *= 80;
-  windowWidth += 35+80;
-  rect = NSMakeRect(100,100,windowWidth,320);
+  windowWidth += 35;
+  rect = NSMakeRect(0,0,windowWidth,320);
 
   _window = [[NSWindow alloc] initWithContentRect:rect
                                         styleMask:style
                                           backing:NSBackingStoreBuffered
                                             defer:YES];
-  [_window setReleasedWhenClosed:YES];
+  [_window setReleasedWhenClosed:NO];
   [_window setMinSize:NSMakeSize(512,320)];
   [_window setDelegate:self];
+  [_window center];
   rect = [[_window contentView] frame];
   
   // Scroll view
@@ -78,6 +78,9 @@
   [_window setContentView:_extScrollView];
   [_window makeFirstResponder:_extEditorView];
   RELEASE(_extScrollView);
+
+  // Honor "edited" state
+  [_window setDocumentEdited:_isEdited];
 }
 
 - (void)_createInternalView
@@ -143,6 +146,10 @@
   [[ev textContainer] setContainerSize:NSMakeSize(fr.size.width, 1e7)];
 
   [ev setEditable:_isEditable];
+
+  // Activate undo
+  [ev setAllowsUndo: YES];
+
   [[NSNotificationCenter defaultCenter]
     addObserver:self 
        selector:@selector(textDidChange:)
@@ -192,6 +199,8 @@
       isCharacterHighlit = NO;
       highlited_chars[0] = -1;
       highlited_chars[1] = -1;
+
+      undoManager = [[NSUndoManager alloc] init];
     }
 
   return self;
@@ -210,6 +219,7 @@
   RELEASE(_categoryPath);
   RELEASE(_intScrollView);
   RELEASE(_storage);
+  RELEASE(_window);
 
 //  RELEASE(parserClasses);
   RELEASE(parserMethods);
@@ -221,6 +231,8 @@
   RELEASE(backgroundColor);
   RELEASE(readOnlyColor);
 
+  RELEASE(undoManager);
+
   [super dealloc];
 }
 
@@ -243,8 +255,6 @@
   NSMutableDictionary *attributes = [NSMutableDictionary new];
   NSFont              *font;
 
-  NSLog(@"PCEditor: openFileAtPath");
-
   // Inform about future file opening
   [[NSNotificationCenter defaultCenter]
     postNotificationName:PCEditorWillOpenNotification
@@ -276,7 +286,7 @@
   [_storage setAttributedString:attributedString];
   RELEASE(attributedString);
 
-  [self _createInternalView];
+//  [self _createInternalView];
 /*  if (categoryPath) // category == nil if we're non project editor
     {
       NSUserDefaults *ud = [NSUserDefaults standardUserDefaults];
@@ -377,11 +387,19 @@
 
 - (NSView *)editorView 
 {
+  if (!_intScrollView)
+    {
+      [self _createInternalView];
+    }
   return _intEditorView;
 }
 
 - (NSView *)componentView
 {
+  if (!_intScrollView)
+    {
+      [self _createInternalView];
+    }
   return _intScrollView;
 }
 
@@ -608,6 +626,12 @@
 	postNotificationName:PCEditorDidSaveNotification
 	  	      object:self];
     }
+  else
+    {
+      NSRunAlertPanel(@"Save File",
+		      @"Couldn't save file '%@'!",
+		      @"OK", nil, nil, [_path lastPathComponent]);
+    }
 
   return saved;
 }
@@ -654,9 +678,31 @@
   return YES;
 }
 
+// FIXME: Do we really need this method?
 - (BOOL)closeFile:(id)sender save:(BOOL)save
 {
-  if ((save == NO) || [self editorShouldClose])
+  if (save == YES)
+    {
+      [self saveFileIfNeeded];
+    }
+
+  // Close window first if visible
+  if (_isWindowed && [_window isVisible] && (sender != _window))
+    {
+      [_window close];
+    }
+
+  // Inform about closing
+  [[NSNotificationCenter defaultCenter] 
+    postNotificationName:PCEditorDidCloseNotification
+		  object:self];
+
+  return YES;
+}
+
+- (BOOL)close:(id)sender
+{
+  if ([self editorShouldClose] == YES)
     {
       // Close window first if visible
       if (_isWindowed && [_window isVisible] && (sender != _window))
@@ -679,7 +725,7 @@
 {
   if (_isEdited)
     {
-      BOOL ret;
+      int ret;
 
       if (_isWindowed && [_window isVisible])
 	{
@@ -687,31 +733,24 @@
 	}
 
       ret = NSRunAlertPanel(@"Close File",
-			    @"File %@ has been modified",
+			    @"File %@ has been modified. Save?",
 			    @"Save and Close", @"Don't save", @"Cancel", 
 			    [_path lastPathComponent]);
-
-      if (ret == YES)
+      switch (ret)
 	{
+	case NSAlertDefaultReturn: // Save And Close
 	  if ([self saveFile] == NO)
 	    {
-	      NSRunAlertPanel(@"Close file",
-		    	      @"Error when saving file '%@'!",
-		    	      @"OK", nil, nil, [_path lastPathComponent]);
 	      return NO;
 	    }
-	  else
-	    {
-	      return YES;
-	    }
-	}
-      else if (ret == NO) // Close but don't save
-	{
-	  return YES;
-	}
-      else               // Cancel closing
-	{
+	  break;
+
+	case NSAlertAlternateReturn: // Don't save
+	  break;
+
+	case NSAlertOtherReturn: // Cancel
 	  return NO;
+	  break;
 	}
 
       [self setIsEdited:NO];
@@ -736,7 +775,7 @@
 	}
       else
 	{
-	  return [self closeFile:_window save:YES];
+    	  return [self close:sender];
 	}
     }
 
@@ -745,20 +784,28 @@
 
 - (void)windowDidBecomeKey:(NSNotification *)aNotification
 {
-  if ([[aNotification object] isEqual:_window] && [_window isVisible])
+/*  if ([[aNotification object] isEqual:_window] && [_window isVisible])
     {
       [_window makeFirstResponder:_extEditorView];
-    }
+    }*/
 }
 
 - (void)windowDidResignKey:(NSNotification *)aNotification
 {
-  if ([[aNotification object] isEqual:_window] && [_window isVisible])
+/*  if ([[aNotification object] isEqual:_window] && [_window isVisible])
     {
-      [_window makeFirstResponder:_window];
-    }
+      [_window makeFirstResponder:_extEditorView];
+    }*/
+  [self resignFirstResponder:_extEditorView];
 }
 
+
+- (NSUndoManager *)windowWillReturnUndoManager:(NSWindow *)window
+{
+  return undoManager;
+}
+
+
 // ===========================================================================
 // ==== TextView (_intEditorView, _extEditorView) delegate
 // ===========================================================================
