@@ -23,13 +23,13 @@
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
 */
 
-#include <ProjectCenter/PCMakefileFactory.h>
-#include <ProjectCenter/PCProjectBrowser.h>
-#include <ProjectCenter/PCFileManager.h>
-#include <ProjectCenter/PCFileCreator.h>
+#import <ProjectCenter/PCMakefileFactory.h>
+#import <ProjectCenter/PCProjectBrowser.h>
+#import <ProjectCenter/PCFileManager.h>
+#import <ProjectCenter/PCFileCreator.h>
 
-#include "PCAppProject.h"
-#include "PCAppProject+Inspector.h"
+#import "PCAppProject.h"
+#import "PCAppProject+Inspector.h"
 
 @implementation PCAppProject
 
@@ -119,8 +119,8 @@
 }
 
 - (void)dealloc
-{
-#ifdef DEVELOPMENT
+{ // TODO: Never called. Should be fixed.
+#ifdef DEBUG
   NSLog (@"PCAppProject: dealloc");
 #endif
 
@@ -256,13 +256,10 @@
 // --- PCProject overridings
 // ----------------------------------------------------------------------------
 
-- (PCProject *)openWithDictionaryAt:(NSString *)path
+- (PCProject *)openWithWrapperAt:(NSString *)path
 {
-  NSDictionary *dict = [NSDictionary dictionaryWithContentsOfFile:path];
-
-  [self assignProjectDict:dict atPath:path];
+  [super openWithWrapperAt: path];
   [self loadInfoFile];
-
   return self;
 }
 
@@ -356,7 +353,7 @@
       [self clearAppIcon:self];
       if ([super renameFile:ff toFile:tf] == YES)
 	{
-	  [self setAppIconWithImageAtPath:
+	  [self setAppIconWithFileAtPath:
 	    [[self dirForCategoryKey:categoryKey] 
 	      stringByAppendingPathComponent:tf]];
 	  success = YES;
@@ -541,6 +538,17 @@
 	[NSString stringWithFormat:@"%@_STANDARD_INSTALL = no\n",
         projectName]];
     }
+  else if (![installDir isEqualToString:@"LOCAL"] &&
+	   ![installDir isEqualToString:@"SYSTEM"] &&
+	   ![installDir isEqualToString:@"USER"] &&
+	   ![installDir isEqualToString:@"NETWORK"] &&
+	   ![installDir isAbsolutePath] &&
+	   [installDir characterAtIndex:0] != '$')
+    {
+      [mff appendString:
+	[NSString stringWithFormat:@"%@_COPY_INTO_DIR = %@\n",
+        projectName, installDir]];
+    }
   else
     {
       /* IMPORTANT FIXME/TODO: We should be using GNUSTEP_INSTALLATION_DOMAIN,
